/*
 * (C) Copyright 2002
 * Richard Jones, rjones@nexus-tech.net
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

/*
 * Boot support
 */
#include <config.h>
#include <common.h>
#include <command.h>
#include <part.h>
#include <fat.h>

int do_fat_fsload (cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	long size;
	int dev=0;
	int part=1;
	char *ep;
	
	block_dev_desc_t *dev_desc=mmc_get_dev(0);

	if (argc < 2) {
		printf ("usage: fatload <dev[:part]> <filename>\n");
		return 1;
	}

	dev = (int)simple_strtoul (argv[1], &ep, 16);
	if (*ep) {
		if (*ep != ':') {
			printf ("\n** Invalid boot device, use `dev[:part]' **\n");
			return 1;
		}
		part = (int)simple_strtoul(++ep, NULL, 16);
	}

	if (fat_register_device(dev_desc,part)!=0) {
		printf ("\n** Unable to use %d:%d for fatload **\n", dev, part);
		return 1;
	}

	size = file_fat_read (argv[2], (u8 *)STAGE2_LOADADDR, 0);

	if(size==-1) {
		printf("\n** Unable to read \"%s\" from %d:%d **\n",
				argv[2], dev, part);
		return 1;
	}

	printf ("\n%ld bytes read\n", size);
	return 0;
}

U_BOOT_CMD(
	fatload,	3,	0,	do_fat_fsload,
	"load binary file from a dos filesystem",
	""
);
