/*
 * Driver for NAND support, Rick Bronson
 * borrowed heavily from:
 * (c) 1999 Machine Vision Holdings, Inc.
 * (c) 1999, 2000 David Woodhouse <dwmw2@infradead.org>
 *
 * Added 16-bit nand support
 * (C) 2004 Texas Instruments
 */

#include <common.h>
#include <command.h>
#include <asm/byteorder.h>

static inline int str2long(char *p, ulong *num)
{
	char *endptr;

	*num = simple_strtoul(p, &endptr, 16);
	return (*p != '\0' && *endptr == '\0') ? 1 : 0;
}

static int
arg_off_size(int argc, char *argv[], ulong *off, size_t *size)
{
	ulong tmp ;

	if (argc >= 1) {
		if (!(str2long(argv[0], off))) {
			printf("'%s' is not a number\n", argv[0]);
			return -1;
		}
	} else {
		*off = 0;
	}
	
	if (argc >= 2) {		
		if (!(str2long(argv[1], &tmp))) {
			printf("'%s' is not a number\n", argv[1]);
			return -1;
		}
		*size = (size_t) tmp;
	} 

	printf("offset 0x%lx, size 0x%x\n", *off, *size);
	return 0;
}

static int nand_dev =0 ;
extern void nand_select_chip(int chip) ;
int nand_read_skip_bad(u32 off, u32 size, uchar* addr) ;

int do_nand(cmd_tbl_t * cmdtp, int flag, int argc, char *argv[])
{
	int ret = 0;
	ulong addr, off;
	size_t size;
	char *cmd;
	int dev ;
	
	if (argc < 2)
		goto usage ;

	cmd = argv[1];

	if (strcmp(cmd, "device") == 0) {

                if (argc < 3) {
			dev = nand_dev ;
		} else {
			dev = (int)simple_strtoul(argv[2], NULL, 10);
			if (dev < 0 || dev > 7) {
				printf("No such device\n");
				return 1;
			}
		}
                printf("Device %d ... is now current device\n", dev);
  
  		nand_dev = dev;
                return 0;
        } else {
		nand_select_chip(nand_dev) ;

		/* at least two arguments please */
		if (argc < 5)
			goto usage;
				
		if (strncmp(cmd, "read", 4) != 0)
			goto usage;
		
		addr = (ulong)simple_strtoul(argv[2], NULL, 16);
		
		printf("\nNAND read: ");
		if (arg_off_size(argc - 3, argv + 3, &off, &size) != 0)
			return 1;
		
		ret = nand_read_skip_bad(off, size, (u_char *)addr);
		
		printf(" %x bytes read: %s\n", size, ret ? "ERROR" : "OK");
		
		return ret == 0 ? 0 : 1;
	}
usage:
	cmd_usage(cmdtp);
	return 1;
}

U_BOOT_CMD(nand, CONFIG_SYS_MAXARGS, 1, do_nand,
	"NAND sub-system",
	"nand read - addr off size\n"
	"nand device [dev] - show or set current device\n"   
);
