/* -------------------------------------------------------------
 * Copyright 2003-2010 Netlogic Microsystems Inc. (“Netlogic”).
 * -------------------------------------------------------------
 * This is a derived work from software originally provided by  
 * the external entity identified below. The licensing terms and  
 * warranties specified in the header of the original work apply 
 * to this derived work.
 * --------------------------#NETL_1#---------------------------
 */
/*
 * (C) Copyright 2008
 * Texas Instruments, <www.ti.com>
 * Syed Mohammed Khasim <khasim@ti.com>
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation's version 2 of
 * the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */
#include <common.h>
#include <config.h>
#include <asm/arch/cpu.h>
#include <part.h>
#include <fat.h>
#include <mmc.h>
#include <mmc_def.h>
#include <mmc_hc.h>

static block_dev_desc_t mmc_blk_dev;

/* INIT_CLOCK - 133.33MHz/(256*2) = 260KHz */
#define MMC_INIT_CLOCK 0x80
/* FAST_CLOCK - 133.33MHz/(8*2)   = 8.33MHz */
#define MMC_FAST_CLOCK 0x800

static int sector_mode = 1;
static int rca = MMC_RCA;
static int is_sd;

/* Low-level write-routines
 */
void hc_wr32(int offset, u32 data, int slot) {

	int hc_offset = (MMC) + (slot*0x100);
	volatile u32 *mmc_base = (u32 *)cpu_io_mmio(0, hc_offset);

	mmc_base[offset>>2] = data;
	return;
}

void hc_wr16(int offset, u16 data, int slot) {

	int hc_offset = (MMC) + (slot*0x100);
	volatile u16 *mmc_base = (u16 *)cpu_io_mmio(0, hc_offset);

	mmc_base[offset>>1] = data;
	return;
}

/* Low-level read-routines
 */
u32 hc_rd32(int offset, int slot) {

	int hc_offset = (MMC) + (slot*0x100);
	volatile u32 *mmc_base = (u32 *)cpu_io_mmio(0, hc_offset);

	u32 data = mmc_base[offset>>2];
	return data;
}

u32 hc_rd16(int offset, int slot) {

	int hc_offset = (MMC) + (slot*0x100);
	volatile u16 *mmc_base = (u16 *)cpu_io_mmio(0, hc_offset);

	u32 data = mmc_base[offset>>1];
	return data;
}

block_dev_desc_t *mmc_get_dev(int dev) {

	return (block_dev_desc_t *) &mmc_blk_dev;
}

static int mmc_cmd(unsigned long cmd,
		unsigned long arg, void *resp) {

	unsigned long *response = resp;
	volatile short intstatus;

	hc_wr32(HC_ARG1_LO, arg,0);
	hc_wr32(HC_COMMAND, cmd,0);
	
	do {
		intstatus = hc_rd32(HC_NORMAL_INT_STS,0);
	} while (!(intstatus & (HNIS_ERR | HNIS_CMD_CMPL)));


	if (intstatus & HNIS_ERR) {
                mmc_dbg("mmc_cmd ERROR\n") ;
                hc_wr32(HC_NORMAL_INT_STS, intstatus,0);
                // Clear CMD inhibit due to error
                hc_wr16(HC_SWRST_TIMEOUT_CTRL,0x20e,0) ;
                while ( (hc_rd16(HC_SWRST_TIMEOUT_CTRL,0) & 0x200) != 0) {} ;
                return -1 ;
        }

	hc_wr32(HC_NORMAL_INT_STS, HNIS_CMD_CMPL,0);

	if ((cmd & RSP_TYPE_MASK) != RSP_TYPE_NORSP) {
		response[0] = hc_rd32(HC_RESPONSE0,0);
		if ((cmd & RSP_TYPE_MASK) == RSP_TYPE_LGHT136) {
			response[1] = hc_rd32(HC_RESPONSE2,0);
			response[2] = hc_rd32(HC_RESPONSE4,0);
			response[3] = hc_rd32(HC_RESPONSE6,0);
		}
	}
	return 0;
}

static int mmc_read_data(u8 *output) {

	volatile short intstatus ;
	int i ;
	volatile uint32_t read_data ;

	do {
		intstatus = hc_rd16(HC_NORMAL_INT_STS, 0) ;
	} while ( !(intstatus & (HNIS_ERR | HNIS_BUFF_RD_RDY) ) );

	if (intstatus & HNIS_ERR) {
		mmc_dbg("mmc_read_data ERROR\n") ;
		return -1 ;
	} 

	hc_wr16(HC_NORMAL_INT_STS, HNIS_BUFF_RD_RDY, 0);

	for (i = 0; i < MMCSD_SECTOR_SIZE / 4; i++) {
		read_data = hc_rd32(HC_BUFF_DATA_PORT0, 0) ;
		output[0] = ( (read_data >>  0)  & 0xff) ;
		output[1] = ( (read_data >>  8)  & 0xff) ;
		output[2] = ( (read_data >>  16) & 0xff) ;
		output[3] = ( (read_data >>  24) & 0xff) ;

		output += 4;	
	}
	return 0;
}

/* Simpler Read routine for directly
 * reading blocks from command line
 */
int mmc_read(u32 start_sec, 
		u32 num_bytes, u8 *output_buf)
{
	u32 argument, resp[4];
	u32 num_sec_val, sec_inc_val;

	num_sec_val = (num_bytes + (MMCSD_SECTOR_SIZE - 1)) / MMCSD_SECTOR_SIZE;

	if (sector_mode) {
		argument = start_sec;
		sec_inc_val = 1;
	} else {
		argument = start_sec * MMCSD_SECTOR_SIZE;
		sec_inc_val = MMCSD_SECTOR_SIZE;
	}

	while (num_sec_val) {
		if (mmc_cmd(MMC_CMD17, argument, resp) ) {
			printf("READ Error: CMD17\n") ;
			return -1; 
		}
		mmc_read_data(output_buf);		
		output_buf += MMCSD_SECTOR_SIZE;	       
		argument   += sec_inc_val;		
		num_sec_val--;
	}
	return 0;
}

/* Main MMC Block read routine, called 
 * through the FAT FS layer code
 */
static unsigned long mmc_read_block(int dev, unsigned long start, 
		lbaint_t blkcnt, void *buffer) {

	u32 argument;
	u32 resp[4];
	u32 num_sec_val;
	u32 sec_inc_val;
	
	u8* output_buffer = (u8 *) buffer;

	if (blkcnt == 0)
		return 0;

	mmc_dbg("mmc_read_block: dev %d, start %lx, blkcnt %lx, blksz %lx\n",
			dev, start, blkcnt, mmc_blk_dev.blksz);

	num_sec_val = blkcnt;

	if (sector_mode) {
		argument = start;
		sec_inc_val = 1;
	} else {
		argument = start * MMCSD_SECTOR_SIZE;
		sec_inc_val = MMCSD_SECTOR_SIZE;
	}

	while (num_sec_val) {

		if (mmc_cmd(MMC_CMD17, argument, resp)) {
			printf("Error! (CMD17)\n");
			return 0;
		}

		mmc_read_data(output_buffer);
		output_buffer += MMCSD_SECTOR_SIZE;
		argument += sec_inc_val;
		num_sec_val--;
	}
	return 1;
}

/* Parse Micron's CID Structure 
 */
static void mmc_parse_cid(struct mmc_cid *cid, u32 *resp)
{
	cid->crc 	= resp[0] & 0xff;
	cid->mdt 	= (resp[0] & 0xff00) >> 8;
	cid->psn 	= ((resp[0] >> 16) & 0xffff) |
				   ((resp[1] & 0xffff) << 16);
	cid->prv 	= (resp[1] >> 16) & 0xff;
	cid->pnm1 	= (resp[1] >> 24) & 0xff;
	cid->pnm2 	= resp[2];
	cid->pnm3 	= resp[3] & 0xff;
	cid->oid 	= (resp[3] & 0xff00) >> 8;
	cid->cbx 	= (resp[3] >> 16) & 0x3;
	cid->mid 	= (resp[3] >> 24) & 0xff;
}

/* Parse SD card's CID Structure 
 */
static void sd_parse_cid(struct mmc_cid *cid, u32 *resp)
{
	printf("SD Card CID: ()\n");
}

static void mmc_dump_csd(const struct mmc_csd *csd)
{
#if 0
	unsigned long *csd_raw = (unsigned long *)csd;
	mmc_dbg("CSD data: %08lx %08lx %08lx %08lx\n",
			csd_raw[0], csd_raw[1], csd_raw[2], csd_raw[3]);
	mmc_dbg("CSD structure version:   1.%u\n", 	csd->csd_structure);
	mmc_dbg("MMC System Spec version: %u\n", 	csd->spec_vers);
	mmc_dbg("Card command classes:    %03x\n", 	csd->ccc);
	mmc_dbg("Read block length:       %u\n", 	1 << csd->read_bl_len);
	if (csd->read_bl_partial) 
		mmc_dbg("Supports partial reads\n");
	else
		mmc_dbg("Does not support partial reads\n");
	mmc_dbg("Write block length:      %u\n", 
			(1 << csd->write_bl_len));
	if (csd->write_bl_partial)
		mmc_dbg("Supports partial writes\n");
	else
		mmc_dbg("Does not support partial writes\n");
	if (csd->wp_grp_enable)
		mmc_dbg("Supports group WP:      %u\n", 	
				csd->wp_grp_size + 1);
	else
		mmc_dbg("Does not support group WP\n");
	mmc_dbg("Card capacity:		%u bytes\n",
			(csd->c_size + 1) * (1 << (csd->c_size_mult + 2)) *
			(1 << csd->read_bl_len));
	mmc_dbg("File format:            %u/%u\n",
			csd->file_format_grp, 
			csd->file_format);
	mmc_dbg("Write protection:        ");
	if (csd->perm_write_protect)
		mmc_dbg(" permanent");
	if (csd->tmp_write_protect)
		mmc_dbg(" temporary");
	mmc_dbg("\n");
#endif
	return;
}

u8 mmc_idle_card(void) {

	/* Issue Idle Command 3 times to
	 * ensure 74 cycles of idle reset
	 */
	int i=0;
	for (i=0; i<3; i++) {
		if (mmc_cmd(MMC_CMD0,0x0,NULL) < 0) {
			printf("Error! (MMC IDLE CMD)\n");
			return -1;
		}
	}
	return 0;
}

static int sd_init_card(struct mmc_cid *cid) {

	int is_sd_v2p = 0;
	u32 resp[4];
	u32 timeout = 1000;
	u32 ocr, ocr_init = OCR_VDD_27_36;

	/*Check!*/
	if (mmc_idle_card() < 0)
		return -1;

	if (mmc_cmd(MMC_SDCMD8, 
		(SD_CMD8_CHECK_PATTERN | SD_CMD8_2_7_3_6_V_RANGE), resp) < 0) {
			mmc_dbg("CMD8 timeout\n");
			is_sd_v2p = 0; 
	} else {
		/* => Card is SD v2.0+ 
		 */
		is_sd_v2p = 1;
		ocr_init  = HCS(1) | ocr_init;

		if ( (resp[0] & 0xff) != 0xaa) {
                        mmc_dbg("CMD8 Check failed\n") ;
                } else {
                        mmc_dbg("CMD8 Check passed\n") ;
                }
	}

	if (mmc_cmd(MMC_CMD55, 0x0, resp) < 0) {
		return -1;
	} else {
		/* Detected SD Card
		 */
		while (1) {

			if (mmc_cmd(MMC_ACMD41, ocr_init, resp) < 0) {
				printf("ACMD41 Error!\n");
				return -1;
			}

			ocr = resp[0];
			if (ocr & OCR_BUSY)
				break;

			if (timeout == 0)
				return -1;

			if (mmc_cmd(MMC_CMD55, 0 , resp) < 0) {
				printf("CMD55 Err\n");
				return -1;
			}
			timeout--;
		}  

		if (ocr & HCS(1)) {
			sector_mode = 1;
			mmc_dbg("SDHC/SDXC: Sector Mode\n");
		} else {
			sector_mode = 0;
			mmc_dbg("SDHC: Byte Mode\n");
		}

		if (mmc_cmd(MMC_CMD2, 0x0, resp) < 0) {
			printf("CMD2 Failed!\n");
			return -1;
		}

		sd_parse_cid(cid, resp);

		if (mmc_cmd(MMC_SDCMD3, 0x0, resp) < 0) {
			printf("CMD3 Failed!\n");
			return -1;
		}
		rca = (resp[0] >> 16);
	}

	return 0;
}

int mmc_init_card(struct mmc_cid *cid) {

	u32 resp[4];
	u32 timeout  = 1000;
	u32 ocr, ocr_init = (OCR_VDD_27_36 | OCR_ACCESSMODE_SECTOR);

	/* Check! */
	if (mmc_idle_card() < 0)
		return -1;

	rca = MMC_RCA;

	while (1) {

		if (mmc_cmd(MMC_CMD1, ocr_init , resp) < 0) {
			printf("CMD1 Err\n");
			return -1;
		} 

		ocr = resp[0];
		if (ocr & OCR_BUSY)
			break;
		if (timeout == 0)
			return -1;
		timeout--;			
	}

	if (ocr & OCR_ACCESSMODE_SECTOR) {
		sector_mode = 1;
		mmc_dbg("Sector Mode\n");
	} else {
		sector_mode = 0;
		mmc_dbg("Byte Mode\n");
	}

	/* Get CID of all cards
	 */
	if (mmc_cmd(MMC_CMD2, 0x0, resp) < 0) {
		printf("MMC CMD2 Error!\n");
		return -1;
	}
	/* print what was found 
	 */
	mmc_parse_cid(cid, resp);

	if (mmc_cmd(MMC_CMD3, (rca << 16), resp) < 0) {
		printf("MMC CMD3 Error!\n");
		return -1;
	}

	return 0;
}

static int mmc_config_card(struct mmc_csd *csd) {

	u32 resp[4];

	/* Int. Clk Enable, SD Clock Disable, Freq */
	hc_wr16(HC_CLOCK_CTRL, HCC_INT_CLK_EN | MMC_INIT_CLOCK, 0);
	hc_wr16(HC_CLOCK_CTRL, HCC_INT_CLK_EN | MMC_FAST_CLOCK, 0);
	hc_wr16(HC_CLOCK_CTRL, HCC_INT_CLK_EN | HCC_SD_CLK_EN | MMC_FAST_CLOCK, 0);

	/* Set RCA, get CSD */
	if (mmc_cmd(MMC_CMD9, (rca << 16), &csd)) {
		printf("Error! (CMD9)\n");
		return -1;
	}

	mmc_dump_csd(csd);

	/* Put Card in Transfer Mode */
	if (mmc_cmd(MMC_CMD7_SELECT, (rca << 16), resp)) {
		printf("Error! (CMD7)\n");
		return -1;
	}

	/* Set sector size to 512 */
	if (mmc_cmd(MMC_CMD16, MMCSD_SECTOR_SIZE, resp)) {
		printf("Error! (CMD16)\n");
		return -1;
	}
	return 0;
}
		
void mmc_init_controller(void) {

	/* Slot 0 Enable & Cache Alloc */
	hc_wr32(HC_SYSCTRL,0x14, 0);

	/* S1: Clear any set INT Bits */
	hc_wr32(HC_NORMAL_INT_STS, hc_rd32(HC_NORMAL_INT_STS,0), 0);

	/* S2: Enable Interrupts */
	hc_wr32(HC_NORMAL_INT_STS_EN, 0x37ff7fff, 0);

	/* S3: Enable Interrupt Signals */
	hc_wr32(HC_NORMAL_INT_SIGNAL_EN, 0x37ff7fff, 0);

	/* S4: Send HW Reset to eMMC-4.4 Card */
	hc_wr16(HC_PC_HC, 0x1e00, 0);
	hc_wr16(HC_PC_HC, 0x1f00, 0);

	/* Int. Clk Enable, SD Clock Disable, Freq */
	hc_wr16(HC_CLOCK_CTRL,(hc_rd16(HC_CLOCK_CTRL,0) & ~(HCC_SD_CLK_EN)), 0);
	hc_wr16(HC_CLOCK_CTRL,HCC_INT_CLK_EN | MMC_INIT_CLOCK, 0);

	/* Wait for stable clock */
	while ((hc_rd16(HC_CLOCK_CTRL,0) & HCC_INT_CLK_STABLE) == 0) {};
	
	/* S4: Remove HW Reset to eMMC-4.4 Card */
	hc_wr16(HC_PC_HC, 0x0f00, 0);

	hc_wr16(HC_CLOCK_CTRL, HCC_INT_CLK_EN | HCC_SD_CLK_EN | MMC_INIT_CLOCK, 0);

	hc_wr16(HC_BLOCK_COUNT, 1, 0);
	hc_wr16(HC_BLOCK_SIZE, MMCSD_SECTOR_SIZE, 0);

        hc_wr16(HC_SWRST_TIMEOUT_CTRL,0xe,0) ;
	printf("Initialized e.MMC Host Controller\n");

	return;
}

int mmc_init(void) {

	struct mmc_cid cid;
	struct mmc_csd csd; 

	/* first, init the controller */
	mmc_init_controller();

	/* then, the card */
	if (sd_init_card(&cid) < 0) {
		if (mmc_init_card(&cid) < 0)
			return -1;
		is_sd = 0;
		printf("Detected MMC Card\n");
	} else {
		printf("Detected SD Card\n");
	}
	
	if (mmc_config_card(&csd) < 0)
		return -1;

	mmc_blk_dev.if_type 	= IF_TYPE_MMC;
	mmc_blk_dev.part_type 	= PART_TYPE_DOS;
	mmc_blk_dev.dev 		= 0;
	mmc_blk_dev.lun 		= 0;
	mmc_blk_dev.type 		= 0;

	/* FIXME fill in the correct size (is set to 32MByte) 
	 */
	mmc_blk_dev.blksz 		= MMCSD_SECTOR_SIZE;
	mmc_blk_dev.lba	 		= 0x10000;
	mmc_blk_dev.removable 	= 0;
	mmc_blk_dev.block_read 	= mmc_read_block;

	fat_register_device(&mmc_blk_dev, 1);

	return 0;
}
