/* Freescale Enhanced Local Bus Controller FCM NAND driver
 *
 * Copyright (c) 2006-2008 Freescale Semiconductor
 *
 * Authors: Nick Spence <nick.spence@freescale.com>,
 *          Scott Wood <scottwood@freescale.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <common.h>
#include <malloc.h>

#include <asm/arch/cpu.h>

struct nand_state {
	u32 cs ;
	u32 col_cyc;
	u32 row_cyc;
	u32 page_size;
	u32 block_size ;
	u32 pages_per_block;
	u32 spare_size ;
	u32 last_cmd ;
	int buf_ptr ;
	u8* buf ;
}  nand_chip ;

static int init_done = 0;
static int waitfunc(void) ;

#if 0
static int wait_ready(volatile u32* mmio, unsigned int rdy_cs_mask, unsigned long timeout)
{
        unsigned int status;

        do {
                status = mmio[NAND_STATUS] ; 

                if ( (status & rdy_cs_mask) == rdy_cs_mask)
                        return 0;

                timeout-- ;
        } while (timeout != 0) ;
	
        printf("nand wait timeout\n") ;

        return -1;
}
#endif

static int dma_wait(volatile u32* mmio,int cs) 
{
	int timeout = 0xfffff ;
	while ( ( (mmio[NAND_STATUS] & (1<<cs) ) == 0) ||
		( (mmio[NAND_DMA_CTRL] & 0x1) == 0) )  {

		timeout--;
		if (timeout == 0) {
			printf("DMA timed out\n") ;
			return -1 ;
		}
	}
	return 0;
}

static unsigned int crc16(unsigned char* data, int length)
{
        // Bit by bit algorithm without augmented zero bytes
        const unsigned int crcinit = 0x4F4E; // Initial CRC value in the shift register
        const int order = 16; // Order of the CRC-16
        const unsigned int polynom = 0x8005; // Polynomial

        unsigned int i, j, c, bit ;
        unsigned int crc = crcinit; // Initialize the shift register with 0x4F4E
        unsigned int crcmask, crchighbit;

        crcmask = ((((unsigned int)1<<(order-1))-1)<<1)|1;
        crchighbit = (unsigned int)1<<(order-1);
        for (i = 0; i < length ; i++)
        {
                c = (unsigned int) (data[i]) ;
                for (j=0x80; j; j>>=1) {
                        bit = crc & crchighbit;
                        crc<<= 1;
                        if (c & j) bit^= crchighbit;
                        if (bit) crc^= polynom;
                }
                crc&= crcmask;
        }
        return crc;
}

void onfi_init(volatile u32* mmio) 
{
	u8* param_ptr = nand_chip.buf ;
	u32 page_val ;
	u32 block_val;
	u32 addr_cyc,addr_val ;
	u32 spare_bytes_per_512 ;
	u32 ecc_bytes ;
	u32 ecc_bits ;
	u32 ecc_val ;
	u32 ecc_offset;
	u32 good_param = 0 ;
	u32 dst_crc ;
	int i;

        for (i = 0; i < 4; i++) {
                dst_crc = (unsigned int) ( (unsigned int) (param_ptr[255] << 8) | (param_ptr[254]) ) ;
                
                if (crc16(param_ptr,254) == dst_crc) {
                        good_param = 1;
                        break ;
                } 
                param_ptr +=  256 ;
        }

        if (good_param == 0) {
                printf("NAND Error: No valid parameter parameter page!\n") ;
                return ;
        } else {
                //printf("NAND found valid parameter parameter page\n") ;
        }

	nand_chip.page_size = ( (unsigned int) (param_ptr[80] << 0)  |
				(unsigned int) (param_ptr[81] << 8)  |
				(unsigned int) (param_ptr[82] << 16) |
				(unsigned int) (param_ptr[83] << 24) ) ;
	
        switch (nand_chip.page_size) {
        case 256:  page_val = 0; break ;
        case 512:  page_val = 1; break ;
        case 1024: page_val = 2; break ;
        case 2048: page_val = 3; break ;
        case 4096: page_val = 4; break ;
        case 8192: page_val = 5; break ;
        case 16384: page_val = 6; break ;
        default: page_val = 7; break ;
        }
	
	nand_chip.pages_per_block  = ( (unsigned int) (param_ptr[92] << 0)  |
				       (unsigned int) (param_ptr[93] << 8)  |
				       (unsigned int) (param_ptr[94] << 16) |
				       (unsigned int) (param_ptr[95] << 24) ) ;
	
        nand_chip.block_size = nand_chip.pages_per_block * nand_chip.page_size ;


        switch (nand_chip.pages_per_block) {
        case 32:  block_val = 0; break ;
        case 64:  block_val = 1; break ;
        case 128: block_val = 2; break ;
        case 256: block_val = 3; break ;
        default: block_val = -1 ; break ;
        }
      
        addr_cyc = param_ptr[101] ;
        nand_chip.row_cyc = (addr_cyc & 0xf) ;
        nand_chip.col_cyc = ( (addr_cyc >> 4) & 0xf) ;
	addr_val = nand_chip.row_cyc + nand_chip.col_cyc ;

        nand_chip.spare_size = ( (unsigned int) (param_ptr[84] << 0)  |
				 (unsigned int) (param_ptr[85] << 8) ) ;
	
        spare_bytes_per_512 = nand_chip.spare_size/(nand_chip.page_size/512) ;

        if (spare_bytes_per_512 <= 4) {
                ecc_bytes = 0 ;
                ecc_bits  = 0 ;
                ecc_val   = 0 ;
        } else if (spare_bytes_per_512 <= 8) {
                ecc_bytes = 4 ;
                ecc_bits  = 2 ;
                ecc_val   = 0 ;
        } else if (spare_bytes_per_512 <= 16) {
                ecc_bytes = 8 ;
                ecc_bits  = 13 ;
                ecc_val   = 3 ;
        } else if (spare_bytes_per_512 <= 24) {
                ecc_bytes = 20 ;
                ecc_bits  = 12 ;
                ecc_val   = 5 ;
        } else {
                ecc_bytes = 23 ;
                ecc_bits  = 14 ;
                ecc_val   = 6 ;
        }
	ecc_offset            = nand_chip.spare_size - ( (nand_chip.page_size/512) * ecc_bytes) ;
    

	mmio[NAND_ECC_CTRL]   = (ecc_val << 5) ;	
        mmio[NAND_ECC_OFFSET] = nand_chip.page_size + ecc_offset ;

        mmio[NAND_CONTROL]    = (mmio[NAND_CONTROL] |
				 NAND_CONTROL_ECC_EN(1)            |
				 NAND_CONTROL_SPARE_EN(1)           |
				 NAND_CONTROL_PAGE_SIZE(page_val)   |
				 NAND_CONTROL_BLOCK_SIZE(block_val) |
				 NAND_CONTROL_ADDR_CYCLE(addr_val) ) ;

	mmio[NAND_SPARE_SIZE] = nand_chip.spare_size - (ecc_bytes * (nand_chip.page_size/512) ) ;
			
}
static void send_cmd(unsigned int command,
		     int column, 
		     int page_addr,
		     int len)
{
	volatile u32* mmio = cpu_io_mmio(0,NAND) ;
 
	mmio[NAND_DATA_SIZE]    = len ;
	mmio[NAND_DMA_CNT]      = len ;
	
	mmio[NAND_DEV_ADDR_0_1] = (page_addr >> (32 - (nand_chip.col_cyc*8) ) ) ;
	mmio[NAND_DEV_ADDR_0_0] = ( (page_addr << (nand_chip.col_cyc * 8) ) | column) ;
	mmio[NAND_DMA_ADDR]     = ( (u32) (nand_chip.buf + nand_chip.buf_ptr) ) & 0x1fffffff ; 
	mmio[NAND_DMA_CTRL]     = (1<<7) | (1<<6) | (5<<2) ;

	if ( (column + len) > nand_chip.page_size) {
		mmio[NAND_CONTROL]      = (mmio[NAND_CONTROL] & ~NAND_CONTROL_ECC_EN(1) ) ;
	}
	
	mmio[NAND_COMMAND]      = command | NAND_CMD_DMA_FLAG;

	dma_wait(mmio,nand_chip.cs) ;	
	mmio[NAND_CONTROL]      = (mmio[NAND_CONTROL] | NAND_CONTROL_ECC_EN(1) ) ;

	nand_chip.last_cmd         = command  ;
}


void nand_select_chip(int chip)
{
	volatile u32* mmio = cpu_io_mmio(0,NAND) ;

	mmio[NAND_MEM_CTRL] = chip ;
	nand_chip.cs        = chip ;
}

static int waitfunc(void)
{
	volatile u32* mmio = cpu_io_mmio(0,NAND) ;
	int timeout = 0xfffff ;

        while ( (mmio[NAND_STATUS] & (1<<nand_chip.cs) ) == 0) {
		timeout--;
		if (timeout == 0) {
			printf("wait func out\n") ;
			return -1 ;
		}
	}

	mmio[NAND_COMMAND] = NAND_READ_STATUS_CMD;
	return mmio[NAND_READ_STATUS] ;
}

int nand_init(void)
{
	volatile u32* mmio = cpu_io_mmio(0,NAND) ;

        mmio[NAND_CONTROL]      = NAND_CONTROL_CUSTOM_XFER_FLAG ;
	mmio[NAND_TIME_SEQ0]    = (NAND_TIME_SEQ0_TWHR(7) | 
				   NAND_TIME_SEQ0_TRHW(7) | 
				   NAND_TIME_SEQ0_TADL(7) | 
				   NAND_TIME_SEQ0_TCCS(7) ) ;

        mmio[NAND_TIMINGS_ASYN] =  (NAND_TIME_ASYN_TRWH(8) | 
				    NAND_TIME_ASYN_TRWP(8) ) ;
    
	return 0;
}

static void nand_read(u32 offset,u32 read_length)
{
	int addr,col,page ;
	//printf("offset %x length %d\n",offset,read_length) ;

	for (addr = offset ; 
	     addr < (offset + read_length);
	     addr = ( (addr & ~(nand_chip.page_size - 1) )  + nand_chip.page_size) ) {
		col  = addr%nand_chip.page_size ;
		page = addr/nand_chip.page_size ;
		//printf("read col %x page %x\n",col,page) ;
		send_cmd(NAND_READ_PAGE_CMD,
			 col, 
			 page,
			 (nand_chip.page_size - col) ) ; 

		nand_chip.buf_ptr = nand_chip.buf_ptr + (nand_chip.page_size - col) ;
	}
     
}
int nand_read_skip_bad(u32 off, u32 size, uchar* addr)
{
	volatile u32* mmio = cpu_io_mmio(0,NAND) ;
	int remain = size ;
	u32 block_offset ;

	nand_chip.buf     = addr ;
	nand_chip.buf_ptr = 0 ;
	if (!init_done) {
		nand_init() ;
		mmio[NAND_COMMAND] = NAND_RESET_CMD ;
		waitfunc() ;
		send_cmd(NAND_READ_PARAMETER_CMD,
			 0, 
			 0,
			 1024) ;
		onfi_init(mmio) ;
		init_done = 1;
	}	
	nand_chip.buf_ptr = 0 ;

	remain = size ;
	while (remain > 0) {
		block_offset = off & (nand_chip.block_size - 1);
			
		send_cmd(NAND_READ_PAGE_CMD,
			 0, 
			 block_offset,
			 (nand_chip.page_size+nand_chip.spare_size) ) ; 
		
		if (nand_chip.buf[nand_chip.page_size] == 0) {
			printf("Bad block %d\n",block_offset) ;
			off += nand_chip.block_size ;
		} else {
			if ( (off + remain) > (off + nand_chip.block_size) ) {
				nand_read(off, nand_chip.block_size - block_offset);
				off    = nand_chip.block_size + (nand_chip.block_size - block_offset) ;
				remain = remain - (nand_chip.block_size - block_offset) ;
			} else {
				nand_read(off, remain);
				remain = 0;
			}
		}
		
	}
	       
	return 0;
}
	
