/*
 * Copyright 2003-2010 Netlogic Microsystems (“Netlogic”). All rights
 * reserved. Redistribution and use in source and binary forms, with 
 * or without modification, are permitted provided that the following 
 * conditions are met:
 *
 *	1. 	Redistributions of source code must retain the above copyright
 *		notice, this list of conditions and the following disclaimer.
 *
 *	2. 	Redistributions in binary form must reproduce the above copyright
 *		notice, this list of conditions and the following disclaimer in
 *		the documentation and/or other materials provided with the
 *		distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY Netlogic Microsystems ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NETLOGIC OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 * --------------------------------#NETL_2#--------------------------------
 */
#ifndef __XLP_DDR_H__
#define __XLP_DDR_H__

#define DDR_NUM_CHANNELS 			4
#define DDR_NUM_RANKS 				4
#define DDR_NUM_DIMMS 				2
#define DDR_NUM_BYTE_LANES 			9

#define DDR_START_DRAM_INIT        	0x04
#define DDR_ENABLE_TX_QUE          	0x80
#define DDR_ENABLE_REFRESH_SCHED   	0x10
#define DDR_ENABLE_POWER_DOWN      	0x20
#define DDR_ENABLE_SELF_REFRESH    	0x40

#define DDR_UCS_NOP      			0x0f
#define DDR_UCS_ZQ       			0x0e
#define DDR_UCS_RD       			0x0d
#define DDR_UCS_WR       			0x0c
#define DDR_UCS_ACTIVE   			0x0b
#define DDR_UCS_PRE      			0x0a
#define DDR_UCS_REF      			0x09
#define DDR_UCS_MRS      			0x08
#define DDR_UCS_PDE      			0x07
#define DDR_UCS_SRE      			0x01

#ifndef __ASSEMBLY__

enum processor_ddr
{
	DDR_SHADOWMR 		= 0,
	DDR_RK0SMR1 		= 1,
	DDR_RK1SMR1 		= 2,
	DDR_RK2SMR1 		= 3,
	DDR_RK3SMR1 		= 4,
	DDR_BP1 			= 5,
	DDR_BP2 			= 6,
	DDR_BP3 			= 7,
	DDR_BP4 			= 8,
	DDR_BP5 			= 9,
	DDR_BP6 			= 10,
	DDR_BP7 			= 11,
	DDR_BP8 			= 12,
	DDR_BP9 			= 13,
	DDR_BP10 			= 14,
	DDR_BP11 			= 15,
	DDR_BP12 			= 16,
	DDR_BP13 			= 17,
	DDR_BP14 			= 18,
	DDR_BP15 			= 19,
	DDR_AP1 			= 20,
	DDR_AP2 			= 21,
	DDR_AP3 			= 22,
	DDR_ODTP1 			= 23,
	DDR_ODTP2 			= 24,
	DDR_PZN1 			= 25,
	DDR_PZN2 			= 26,
	DDR_PZN3 			= 27,
	DDR_PZN4 			= 28,
	DDR_ECCLOG1 		= 29,
	DDR_ECCLOG2 		= 30,
	DDR_DIS1 			= 31,
	DDR_DIS2 			= 32,
	DDR_DIS3 			= 33,
	DDR_DIS4 			= 34,
	DDR_DIS5 			= 35,
	DDR_DIS6 			= 36,
	DDR_RFS1 			= 37,
	DDR_RFS2 			= 38,
	DDR_RFS3 			= 39,
	DDR_RFS4 			= 40,
	DDR_SRE1 			= 41,
	DDR_SRE2 			= 42,
	DDR_SRE3 			= 43,
	DDR_SRE4 			= 44,
	DDR_ZQC1 			= 45,
	DDR_ZQC2 			= 46,
	DDR_PDE1 			= 47,
	DDR_PDE2 			= 48,
	DDR_PDX1 			= 49,
	DDR_PDX2 			= 50,
	DDR_UCS1 			= 51,
	DDR_UCS2 			= 52,
	DDR_UCS3 			= 53,
	DDR_UCS4 			= 54,
	DDR_UCS5 			= 55,
	DDR_UCS6 			= 56,
	DDR_DP1 			= 57,
	DDR_DP2 			= 58,
	DDR_DP3 			= 59,
	DDR_DP4 			= 60,
	DDR_DP5 			= 61,
	DDR_DP6 			= 62,
	DDR_DP7 			= 63,
	DDR_DP8 			= 64,
	DDR_DP9 			= 65,
	DDR_DP10 			= 66,
	DDR_DP11 			= 67,
	DDR_DP12 			= 68,
	DDR_DP13 			= 69,
	DDR_DP14 			= 70,
	DDR_DP15 			= 71,
	DDR_NSL 			= 72,
	DDR_CM0 			= 73,
	DDR_CM1 			= 74,
	DDR_UCSC 			= 75,
	DDR_WLLIM 			= 76,
	DDR_WLCTL 			= 77,
	DDR_CST 			= 78,
	DDR_GPCE 			= 79,
	DDR_PCC0 			= 80,
	DDR_PCV0 			= 81,
	DDR_PCC1 			= 82,
	DDR_PCV1 			= 83,
	DDR_PCC2 			= 84,
	DDR_PCV2 			= 85,
	DDR_PCC3 			= 86,
	DDR_PCV3 			= 87,
	DDR_TGED1 			= 88,
	DDR_TGED2 			= 89,
	DDR_TGED3 			= 90,
	DDR_TGED4 			= 91,
	DDR_TGED5 			= 92,
	DDR_TGED6 			= 93,
	DDR_TGED7 			= 94,
	DDR_TGED8 			= 95,
	DDR_TGED9 			= 96,
	DDR_TGED10 			= 97,
	DDR_TGED11 			= 98,
	DDR_TGED12 			= 99,
	DDR_TGED13 			= 100,
	DDR_TGED14 			= 101,
	DDR_TGED15 			= 102,
	DDR_TGED16 			= 103,
	DDR_TGECB1 			= 104,
	DDR_TGECB2 			= 105,
	DDR_TGEA1 			= 106,
	DDR_TGEA2 			= 107,
	DDR_TGECNT 			= 108,
	DDR_TGE_RDRETCNT 	= 109,
	DDR_TGE_DMVECTLO 	= 110,
	DDR_TGE_DMVECTHI 	= 111,
	DDR_TGE_CBMVECT 	= 112,
	DDR_TGECON 			= 113,
	DDR_SCR0 			= 114,
	DDR_SCR1 			= 115,
	DDR_SCR2 			= 116,
	DDR_SCR3 			= 117,
	DDR_PRG 			= 118
};

enum processor_ddr_phy
{
	/* DDR PHY DLL BASE REGISTERS 
	 */
	DDR_PHY_DQS0_CALDLL_BASE  		= 0,
	DDR_PHY_DQS1_CALDLL_BASE  		= 64,
	DDR_PHY_DQS2_CALDLL_BASE  		= 128,
	DDR_PHY_DQS3_CALDLL_BASE  		= 192,
	DDR_PHY_DQS4_CALDLL_BASE  		= 256,
	DDR_PHY_DQS5_CALDLL_BASE  		= 320,
	DDR_PHY_DQS6_CALDLL_BASE  		= 384,
	DDR_PHY_DQS7_CALDLL_BASE  		= 448,
	DDR_PHY_DQS8_CALDLL_BASE  		= 512,
	DDR_PHY_MRCMD_CALDLL_BASE 		= 576, 
	DDR_PHY_PAD0_CAL_BASE     		= 640,
	DDR_PHY_PAD1_CAL_BASE     		= 704,
	DDR_PHY_PAD2_CAL_BASE     		= 768,
	DDR_PHY_SRCMD_CALDLL_BASE 		= 832,

	/* DDR PHY DQS OFFSETS 
	 */
	DDR_PHY_DQS_CALDLL_RES         	= 0,
	DDR_PHY_DQS_SHR_TXPSDLL_CTRL   	= 1,
	DDR_PHY_DQS_RK0_TXWLDLL_CTRL   	= 2,
	DDR_PHY_DQS_RK1_TXWLDLL_CTRL   	= 3,
	DDR_PHY_DQS_RK2_TXWLDLL_CTRL   	= 4,
	DDR_PHY_DQS_RK3_TXWLDLL_CTRL   	= 5,
	DDR_PHY_DQS_RK0_RXSEDLL_CTRL   	= 6,
	DDR_PHY_DQS_RK1_RXSEDLL_CTRL   	= 7,
	DDR_PHY_DQS_RK2_RXSEDLL_CTRL   	= 8,
	DDR_PHY_DQS_RK3_RXSEDLL_CTRL   	= 9,
	DDR_PHY_DQS_RK0_RXPSDLL_CTRL   	= 10,
	DDR_PHY_DQS_RK1_RXPSDLL_CTRL   	= 11,
	DDR_PHY_DQS_RK2_RXPSDLL_CTRL   	= 12,
	DDR_PHY_DQS_RK3_RXPSDLL_CTRL   	= 13,
	DDR_PHY_DQS_SHR_WLRSLT_DATA_LO 	= 14,
	DDR_PHY_DQS_SHR_WLRSLT_DATA_HI 	= 15,
	DDR_PHY_DQS_SHR_WLRSLT_VLD_LO 	= 16,
	DDR_PHY_DQS_SHR_WLRSLT_VLD_HI 	= 17,
	DDR_PHY_DQS_SHR_WLRSLT_PURE_LO 	= 18,
	DDR_PHY_DQS_SHR_WLRSLT_PURE_HI 	= 19,
	DDR_PHY_DQS_SHR_WLED_CFG 		= 20,
	DDR_PHY_DQS_SHR_WLED_FVR 		= 21,
	DDR_PHY_DQS_SHR_WLED_NVR 		= 22,
	DDR_PHY_DQS_SHR_WLED_ALN 		= 23,
	DDR_PHY_DQS_DQ0_MICRODLL_CTRL 	= 24,
	DDR_PHY_DQS_DQ1_MICRODLL_CTRL 	= 25,
	DDR_PHY_DQS_DQ2_MICRODLL_CTRL 	= 26,
	DDR_PHY_DQS_DQ3_MICRODLL_CTRL 	= 27,
	DDR_PHY_DQS_DQ4_MICRODLL_CTRL 	= 28,
	DDR_PHY_DQS_DQ5_MICRODLL_CTRL 	= 29,
	DDR_PHY_DQS_DQ6_MICRODLL_CTRL 	= 30,
	DDR_PHY_DQS_DQ7_MICRODLL_CTRL 	= 31,
	DDR_PHY_DQS_RK0_TXWLDLY_SEL 	= 32,
	DDR_PHY_DQS_RK1_TXWLDLY_SEL 	= 33,
	DDR_PHY_DQS_RK2_TXWLDLY_SEL 	= 34,
	DDR_PHY_DQS_RK3_TXWLDLY_SEL 	= 35,
	DDR_PHY_DQS_RK0_RXSEDLY_SEL 	= 36,
	DDR_PHY_DQS_RK1_RXSEDLY_SEL 	= 37,
	DDR_PHY_DQS_RK2_RXSEDLY_SEL 	= 38,
	DDR_PHY_DQS_RK3_RXSEDLY_SEL 	= 39,
	DDR_PHY_DQS_RK0_RXDVDLY_SEL 	= 40,
	DDR_PHY_DQS_RK1_RXDVDLY_SEL 	= 41,
	DDR_PHY_DQS_RK2_RXDVDLY_SEL 	= 42,
	DDR_PHY_DQS_RK3_RXDVDLY_SEL 	= 43,
	DDR_PHY_DQS_RK0_CTENDLY_SEL 	= 44,
	DDR_PHY_DQS_RK1_CTENDLY_SEL 	= 45,
	DDR_PHY_DQS_RK2_CTENDLY_SEL 	= 46,
	DDR_PHY_DQS_RK3_CTENDLY_SEL 	= 47,
	DDR_PHY_DQS_STRB_EN_SAMPLE 		= 48,
	DDR_PHY_DQS_PAD_DRV_SCALE 		= 49,
	DDR_PHY_DQS_PAD_TERM_SCALE 		= 50,
	DDR_PHY_DQS_PAD_DRV_CTRL 		= 51,
	DDR_PHY_DQS_PAD_TERM_CTRL 		= 52,
};

#endif /* __ASSEMBLY__ */
#endif /* __XLP_DDR_H__ */
