#ifndef _XLP_NAND_H
#define _XLP_NAND_H

#define NAND_COMMAND_REG   0
#define NAND_CONTROL_REG   1

#ifndef __ASSEMBLY__
enum processor_nand
{
	NAND_COMMAND        = 0,
	NAND_CONTROL        = 1,
	NAND_STATUS         = 2,
	NAND_INT_MASK       = 3,
	NAND_INT_STATUS     = 4,
	NAND_ECC_CTRL       = 5,
	NAND_ECC_OFFSET     = 6,
	NAND_DEV_ADDR_0_0   = 7,
	NAND_DEV_ADDR_1_0   = 8,
	NAND_DEV_ADDR_0_1   = 9,
	NAND_DEV_ADDR_1_1   = 10,
	NAND_SPARE_SIZE     = 11,
	NAND_DMA_ADDR       = 25,
	NAND_DMA_CNT        = 26,
	NAND_DMA_CTRL       = 27,
	NAND_MEM_CTRL       = 32,
	NAND_DATA_SIZE      = 33,
	NAND_READ_STATUS    = 34,
	NAND_TIME_SEQ0      = 35,
	NAND_TIMINGS_ASYN   = 36,
	NAND_TIMINGS_SYN    = 37,
	NAND_FIFO_DATA      = 38,
	NAND_FIFO_STATE     = 46,
	NAND_TIME_SEQ1      = 47

};
#endif
//                                     CMD 3         CMD 2          CMD 1         SEQ
#define NAND_RESET_CMD              ( (0x0  << 24) | (0x0  << 16) | (0xFF << 8) | 0x0) 
#define NAND_READ_PARAMETER_CMD     ( (0x0  << 24) | (0x0  << 16) | (0xEC << 8) | 0x22) 
#define NAND_READ_ID_CMD            ( (0x0  << 24) | (0x0  << 16) | (0x90 << 8) | 0x21) 
#define NAND_READ_PAGE_CMD          ( (0x0  << 24) | (0x30 << 16) | (0x00 << 8) | 0x2a) 
#define NAND_ERASE_BLOCK_CMD        ( (0x0  << 24) | (0xD0 << 16) | (0x60 << 8) | 0xe) 
#define NAND_PAGE_PROGRAM_CMD       ( (0x0  << 24) | (0x10 << 16) | (0x80 << 8) | 0xc) 
#define NAND_READ_STATUS_CMD        ( (0x0  << 24) | (0x00 << 16) | (0x70 << 8) | 0x24) 

#define NAND_CMD_DMA_FLAG   (1<<6)
#define NAND_CMD_ADDR1_FLAG (1<<7)

#define NAND_CONTROL_X16_FLAG (1<<12)
#define NAND_CONTROL_CUSTOM_XFER_FLAG (1<<11)
#define NAND_CONTROL_PAGE_SIZE(size) (size<<8)
#define NAND_CONTROL_BLOCK_SIZE(size) (size<<6)
#define NAND_CONTROL_ADDR_CYCLE(cyc) (cyc<<0)
#define NAND_CONTROL_ECC_EN(en) (en<<5)
#define NAND_CONTROL_SPARE_EN(en) (en<<3)

/*Sync mode WE High->RE Low*/
#define NAND_TIME_SEQ0_TWHR(x) (x<<24)
/*ASync mode RE High->WE Low*/
#define NAND_TIME_SEQ0_TRHW(x) (x<<16)
/*Async ALE->Data start*/
#define NAND_TIME_SEQ0_TADL(x) (x<<8)
/*Chance column setup*/
#define NAND_TIME_SEQ0_TCCS(x) (x<<0)
/*TRR time peroid*/
#define NAND_TIME_SEQ1_TRR(x) (x<<9)
/*Busy time peroid for async->sync*/
#define NAND_TIME_SEQ1_TWB(x) (x<<0)
/*RE/WE high hold time*/
#define NAND_TIME_ASYN_TRWH(x) (x<<4)
/*RE/WE pulse width*/
#define NAND_TIME_ASYN_TRWP(x) (x<<0)

#endif
