/*
 * Copyright 2003-2010 Netlogic Microsystems (“Netlogic”). All rights
 * reserved. Redistribution and use in source and binary forms, with 
 * or without modification, are permitted provided that the following 
 * conditions are met:
 *
 *	1. 	Redistributions of source code must retain the above copyright
 *		notice, this list of conditions and the following disclaimer.
 *
 *	2. 	Redistributions in binary form must reproduce the above copyright
 *		notice, this list of conditions and the following disclaimer in
 *		the documentation and/or other materials provided with the
 *		distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY Netlogic Microsystems ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NETLOGIC OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 * --------------------------------#NETL_2#--------------------------------
 */
#ifndef __XLP_SPI_H_
#define __XLP_SPI_H_

#ifndef __ASSEMBLY__
enum processor_spi
{
	CS0_CONFIG      = 0,
	CS0_FDIV        = 1,
	CS0_CMD         = 2,
	CS0_STATUS      = 3,
	CS0_INTEN       = 4,
	CS0_FIFO_THRESH = 5,
	CS0_FIFO_WCNT   = 6,
	CS0_TXDATA_FIFO = 7,
	CS0_RXDATA_FIFO = 8,

	CS1_CONFIG      = 16,
	CS1_FDIV        = 17,
	CS1_CMD         = 18,
	CS1_STATUS      = 19,
	CS1_INTEN       = 20,
	CS1_FIFO_THRESH = 21,
	CS1_FIFO_WCNT   = 22,
	CS1_TXDATA_FIFO = 23,
	CS1_RXDATA_FIFO = 24,

	CS2_CONFIG      = 32,
	CS2_FDIV        = 33,
	CS2_CMD         = 34,
	CS2_STATUS      = 35,
	CS2_INTEN       = 36,
	CS2_FIFO_THRESH = 37,
	CS2_FIFO_WCNT   = 38,
	CS2_TXDATA_FIFO = 39,
	CS2_RXDATA_FIFO = 40,

	CS3_CONFIG      = 48,
	CS3_FDIV        = 49,
	CS3_CMD         = 50,
	CS3_STATUS      = 51,
	CS3_INTEN       = 52,
	CS3_FIFO_THRESH = 53,
	CS3_FIFO_WCNT   = 54,
	CS3_TXDATA_FIFO = 55,
	CS3_RXDATA_FIFO = 56,

	SPI_SYSCNTRL    = 64	
};
#endif /* __ASSEMBLY__ */

/* Simple flags to differentiate 
 * between driver instances 
 */
#define SPINAND					0
#define SPINOR					1

#define CS_OFFSET 				(CS1_CONFIG - CS0_CONFIG)

#define SPI_SYSCNTRL_RESET_CS0 	0x1
#define SPI_SYSCNTRL_RESET_CS1 	0x2
#define SPI_SYSCNTRL_RESET_CS2 	0x4
#define SPI_SYSCNTRL_RESET_CS3 	0x8
#define SPI_SYSCNTRL_CLKDIS_CS0 0x10
#define SPI_SYSCNTRL_CLKDIS_CS1 0x20
#define SPI_SYSCNTRL_CLKDIS_CS2 0x40
#define SPI_SYSCNTRL_CLKDIS_CS3 0x80
#define SPI_SYSCNTRL_ENABLE 	0x100

#define SPI_CMD_IDLE  			0x0
#define SPI_CMD_TX(x) 			x
#define SPI_CMD_RX(x) 			(x<<1)
#define SPI_CMD_TXRX  			0x3
#define SPI_CMD_CONT(x)  		(x<<4)
#define SPI_CMD_XFERCNT(CNT) 	(CNT << 16) 

#define SPI_STATUS_PEND     	0x1
#define SPI_STATUS_DONE     	0x2
#define SPI_STATUS_TXTHRESH 	0x4
#define SPI_STATUS_RXTHRESH 	0x8

#define SPI_THRESH_RX(thresh) 	thresh
#define SPI_THRESH_TX(thresh) 	(thresh << 4)

#define SPI_FIFO_SIZE 8
#define SPI_TXFIFO_CNT(cnt) 	((cnt >> 4) & 0xf)
#define SPI_RXFIFO_CNT(cnt) 	(cnt & 0xf)

#define SPI_CFG_CPHA(x)   		(x << 0)
#define SPI_CFG_CPOL(x)   		(x << 1)
#define SPI_CFG_CSPOL(x)  		(x << 2)
#define SPI_CFG_TXMISO(x) 		(x << 3)
#define SPI_CFG_TXMOSI(x) 		(x << 4)
#define SPI_CFG_RXMISO(x) 		(x << 5)
#define SPI_CFG_SBE(x)    		(x << 6)
#define SPI_CFG_SBPOL(x)  		(x << 7)
#define SPI_CFG_CSTODATA(x) 	(x << 8)
#define SPI_CFG_LSBFE(x)  		(x << 10)
#define SPI_CFG_RXCAP(x)  		(x << 11)

#endif /* __XLP_SPI_H_ */
