/*
 * Copyright 2008, Freescale Semiconductor, Inc
 * Andy Fleming
 *
 * Based (loosely) on the Linux code
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#ifndef _MMC_DEF_H_
#define _MMC_DEF_H_

#ifdef  MMC_DBG
#define mmc_dbg(fmt,args...)     printf(fmt,##args)
#else
#define mmc_dbg(fmt,args...)
#endif

#define CMD(i)   (i << 24)

#define RSP_TYPE_MASK                   (0x3 << 16)
#define RSP_TYPE_NORSP                  (0x0 << 16)
#define RSP_TYPE_LGHT136                (0x1 << 16)
#define RSP_TYPE_LGHT48                 (0x2 << 16)
#define RSP_TYPE_LGHT48B                (0x3 << 16)
#define CCCE_NOCHECK                    (0x0 << 19)
#define CCCE_CHECK                      (0x1 << 19)
#define CICE_NOCHECK                    (0x0 << 20)
#define CICE_CHECK                      (0x1 << 20)
#define DP_NO_DATA                      (0x0 << 21)
#define DP_DATA                         (0x1 << 21)
#define DDIR_WRITE                      (0x0 << 4)
#define DDIR_READ                       (0x1 << 4)

/* Responses */
#define RSP_TYPE_NONE   (RSP_TYPE_NORSP   | CCCE_NOCHECK | CICE_NOCHECK)
#define RSP_TYPE_R1     (RSP_TYPE_LGHT48  | CCCE_CHECK   | CICE_CHECK)
#define RSP_TYPE_R1B    (RSP_TYPE_LGHT48B | CCCE_CHECK   | CICE_CHECK)
#define RSP_TYPE_R2     (RSP_TYPE_LGHT136 | CCCE_CHECK   | CICE_NOCHECK)
#define RSP_TYPE_R3     (RSP_TYPE_LGHT48  | CCCE_NOCHECK | CICE_NOCHECK)
#define RSP_TYPE_R4     (RSP_TYPE_LGHT48  | CCCE_NOCHECK | CICE_NOCHECK)
#define RSP_TYPE_R5     (RSP_TYPE_LGHT48  | CCCE_CHECK   | CICE_CHECK)
#define RSP_TYPE_R6     (RSP_TYPE_LGHT48  | CCCE_CHECK   | CICE_CHECK)
#define RSP_TYPE_R7     (RSP_TYPE_LGHT48  | CCCE_CHECK   | CICE_CHECK)

#define MMC_CMD0        (CMD(0)  | RSP_TYPE_NONE | DP_NO_DATA | DDIR_WRITE)
#define MMC_CMD1        (CMD(1)  | RSP_TYPE_R3   | DP_NO_DATA | DDIR_WRITE)
#define MMC_CMD2        (CMD(2)  | RSP_TYPE_R2   | DP_NO_DATA | DDIR_WRITE)
#define MMC_CMD3        (CMD(3)  | RSP_TYPE_R1   | DP_NO_DATA | DDIR_WRITE)
#define MMC_SDCMD3      (CMD(3)  | RSP_TYPE_R6   | DP_NO_DATA | DDIR_WRITE)
#define MMC_CMD4        (CMD(4)  | RSP_TYPE_NONE | DP_NO_DATA | DDIR_WRITE)
#define MMC_CMD6        (CMD(6)  | RSP_TYPE_R1B  | DP_NO_DATA | DDIR_WRITE)
#define MMC_CMD7_SELECT (CMD(7)  | RSP_TYPE_R1B  | DP_NO_DATA | DDIR_WRITE)
#define MMC_CMD7_DESELECT (CMD(7)| RSP_TYPE_NONE | DP_NO_DATA | DDIR_WRITE)
#define MMC_CMD8        (CMD(8)  | RSP_TYPE_R1   | DP_DATA    | DDIR_READ)
#define MMC_SDCMD8      (CMD(8)  | RSP_TYPE_R7   | DP_NO_DATA | DDIR_WRITE)
#define MMC_CMD9        (CMD(9)  | RSP_TYPE_R2   | DP_NO_DATA | DDIR_WRITE)
#define MMC_CMD12       (CMD(12) | RSP_TYPE_R1B  | DP_NO_DATA | DDIR_WRITE)
#define MMC_CMD13       (CMD(13) | RSP_TYPE_R1   | DP_NO_DATA | DDIR_WRITE)
#define MMC_CMD15       (CMD(15) | RSP_TYPE_NONE | DP_NO_DATA | DDIR_WRITE)
#define MMC_CMD16       (CMD(16) | RSP_TYPE_R1   | DP_NO_DATA | DDIR_WRITE)
#define MMC_CMD17       (CMD(17) | RSP_TYPE_R1   | DP_DATA    | DDIR_READ)
#define MMC_CMD24       (CMD(24) | RSP_TYPE_R1   | DP_DATA    | DDIR_WRITE)
#define MMC_ACMD6       (CMD(6)  | RSP_TYPE_R1   | DP_NO_DATA | DDIR_WRITE)
#define MMC_ACMD41      (CMD(41) | RSP_TYPE_R3   | DP_NO_DATA | DDIR_WRITE)
#define MMC_ACMD51      (CMD(51) | RSP_TYPE_R1   | DP_DATA    | DDIR_READ)
#define MMC_CMD55       (CMD(55) | RSP_TYPE_R1   | DP_NO_DATA | DDIR_WRITE)

/* High Capacity Support */
#define HCS(x) (x << 30)

#define MMC_HS_TIMING					0x00000100
#define MMC_HS_52MHZ					0x2

#define SD_CMD8_CHECK_PATTERN   0xAA
#define SD_CMD8_2_7_3_6_V_RANGE (0x01 << 8)

/* OCR Register Bit Definitions
 */
#define OCR_BUSY						0x80000000
#define OCR_ACCESSMODE_SECTOR			0x40000000
#define OCR_VDD_165_195					0x00000080	/* VDD volt 1.65 - 1.95 */
#define OCR_VDD_27_36					0x00FF8000	/* VDD volt 2.70 ~ 3.60 */

#define MMC_SWITCH_MODE_CMD_SET			0x00 /* Change the command set */
#define MMC_SWITCH_MODE_SET_BITS		0x01 /* Set bits in EXT_CSD byte
												addressed by index which are
												1 in value field */
#define MMC_SWITCH_MODE_CLEAR_BITS		0x02 /* Clear bits in EXT_CSD byte
												addressed by index, which are
												1 in value field */
#define MMC_SWITCH_MODE_WRITE_BYTE		0x03 /* Set target byte to value */

#endif /* _MMC_H_ */
