/*
 * Copyright 2003-2010 Netlogic Microsystems (“Netlogic”). All rights
 * reserved. Redistribution and use in source and binary forms, with 
 * or without modification, are permitted provided that the following 
 * conditions are met:
 *
 *	1. 	Redistributions of source code must retain the above copyright
 *		notice, this list of conditions and the following disclaimer.
 *
 *	2. 	Redistributions in binary form must reproduce the above copyright
 *		notice, this list of conditions and the following disclaimer in
 *		the documentation and/or other materials provided with the
 *		distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY Netlogic Microsystems ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NETLOGIC OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 * --------------------------------#NETL_2#--------------------------------
 */
#ifndef __MMC_HC__
#define __MMC_HC__

/* Core Arasan Host Controller Registers
 */
#define HC_SDMA_SA_OR_ARG2_LO 		0x0000 
#define HC_SDMA_SA_OR_ARG2_HI 		0x0002 
#define HC_BLOCK_SIZE 				0x0004
#define HC_BLOCK_COUNT 				0x0006
#define HC_ARG1_LO 					0x0008
#define HC_ARG1_HI  				0x000A
#define HC_TRANSFER_MODE 			0x000C
#define HC_COMMAND 					0x000E
#define HC_DATA_PRESENT_SEL		0x0020
#define HC_CMD_INDEX_CHK_EN		0x0010
#define HC_CMD_CRC_CHK_EN		0x0008

#define HC_RESPONSE0 				0x0010
#define HC_RESPONSE1 				0x0012
#define HC_RESPONSE2 				0x0014
#define HC_RESPONSE3 				0x0016
#define HC_RESPONSE4 				0x0018
#define HC_RESPONSE5 				0x001A
#define HC_RESPONSE6 				0x001C
#define HC_RESPONSE7 				0x001E

#define HC_BUFF_DATA_PORT0 		0x0020
#define HC_BUFF_DATA_PORT1 		0x0022
#define HC_PRESENT_STATE_LO 		0x0024
#define HC_PRESENT_STATE_HI 		0x0026
#define HC_PC_HC 		      	0x0028
#define HC_WC_BGC		       	0x002A
#define HC_CLOCK_CTRL 		       	0x002C
#define HCC_INT_CLK_EN 			0x0001
#define HCC_INT_CLK_STABLE              0x0002
#define HCC_SD_CLK_EN 			0x0004
#define HC_SWRST_TIMEOUT_CTRL 		0x002E

#define HC_NORMAL_INT_STS 			0x0030
#define HNIS_CMD_CMPL 			0x0001 /* command completion intr. */
#define HNIS_TC_CMPL 			0x0002 /* transaction completion intr. */
#define HNIS_BUFF_RD_RDY		0x0020 /* ready to read buffer intr. */
#define HNIS_ERR 			0x8000 /* command completion intr. */
#define HC_ERROR_INT_STS 			0x0032
#define HC_NORMAL_INT_STS_EN 		0x0034
#define HC_ERROR_INT_STS_EN 		0x0036
#define HC_NORMAL_INT_SIGNAL_EN 	0x0038
#define HC_ERROR_INT_SIGNAL_EN 		0x003A
#define HC_AUTO_CMD_ERROR_STS 		0x003C
#define HC_HOST_CTRL_2 		       	0x003E

#define HC_CAP0_LO     			0x0040
#define HC_CAP0_HI 	       		0x0042
#define HC_CAP1_LO 	      		0x0044 
#define HC_CAP1_RSVD 			0x0046
#define HC_MAX_CURRENT_CAP_LO 		0x0048 
#define HC_MAX_CURRENT_CAP_HI 		0x004A 

#define HC_ADMA_SA_15_0  			0x0058 
#define HC_ADMA_SA_31_16 			0x005A 
#define HC_ADMA_SA_47_32 			0x005C 
#define HC_ADMA_SA_63_48 			0x005E 

#define HC_BOOTDATA_TIMEOUT_CTRL_LO 0x0070 
#define HC_BOOTDATA_TIMEOUT_CTRL_HI 0x0072 

#define HC_SHARED_BUS_CTRL_LO 		0x00E0 
#define HC_SHARED_BUS_CTRL_HI 		0x00E2 

#define HC_HOST_CONTROLLER_VERSION 	0x00FE 
#define HC_SPI_INT_SUPPORT 	       	0x00F0 
#define HC_SLOT_INT_STS 	       	0x00FC 

#define HC_SYSCTRL 	       	        0x0200 

/*
 */
#define MMC_DEFAULT_RCA			0x1
#define MMC_RCA		        	0xA
#define MMCSD_SECTOR_SIZE               512

/* Host Controller Command-Response defines
 */
#define RSP_NONE    				0x0
#define RSP_136    			       	0x1
#define RSP_48    			       	0x2
#define RSP_48_CHKBUSY				0x3

#define R1		(RSP_48 | HC_CMD_INDEX_CHK_EN | HC_CMD_CRC_CHK_EN)
#define R1b 	        (RSP_48_CHKBUSY | HC_CMD_INDEX_CHK_EN | HC_CMD_CRC_CHK_EN)
#define R2		(RSP_136 | HC_CMD_CRC_CHK_EN)
#define R3 		RSP_48
#define R4		R3
#define R5		R1
#define R5b		R1b
#define R6   	R1
#define R7		R1

#endif /* __MMC_HC__ */
