/* -------------------------------------------------------------
 * Copyright 2003-2010 Netlogic Microsystems Inc. (“Netlogic”).
 * -------------------------------------------------------------
 * This is a derived work from software originally provided by  
 * the external entity identified below. The licensing terms and  
 * warranties specified in the header of the original work apply 
 * to this derived work.
 * --------------------------#NETL_1#---------------------------
 */
/*
 * Copyright (C) 2005 Texas Instruments.
 *
 * (C) Copyright 2004
 * Jian Zhang, Texas Instruments, jzhang@ti.com.
 *
 * (C) Copyright 2002
 * Wolfgang Denk, DENX Software Engineering, wd@denx.de.
 *
 * (C) Copyright 2002
 * Sysgo Real-Time Solutions, GmbH <www.elinos.com>
 * Marius Groeger <mgroeger@sysgo.de>
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>
#include <part.h>
#include <fat.h>
#include <mmc.h>
#include <asm/arch/cpu.h>

#define __raw_readl(a)	(*(volatile unsigned int *)(a))

extern char * strcpy(char * dest,const char *src);

/*
 * A0 Masking of bogus failing BIST controllers
 */
static const u32 bist_mask[] = {0x01000b00,
				0x2c020016,
				0x00580400,
				0x1000b008,
				0xc0200160,
				0x05804002,
				0x00000080,
				0x80000000,
				0x00000000,
				0x00000000,
				0x00000000,
				0x00000000,
				0xf0000000};

int print_info(void)
{
	printf ("\n\nNetLogic XLP Stage 1 Loader("
		__DATE__ " - " __TIME__ ")\n");

	volatile u32* mmio = cpu_io_mmio(0,SYS) ;
	int i;
	int failed = 0;
	for (i=0;i<13;i++) {
		if ( (mmio[SYS_SYSMEMBISTGO0 + i] | bist_mask[i]) != 0xffffffff) {
			printf ("*** MBist%d Fail: 0x%08x ***\n",i,mmio[SYS_SYSMEMBISTGO0 + i]) ;
			failed = 1 ;
		}
	}
	if (!failed) {
		printf("MBist Pass\n") ;
	}
	return 0;
}

typedef int (init_fnc_t) (void);

/* Note: Maintain init order!
 */
init_fnc_t *init_sequence[] = {
	cpu_init,			/* basic cpu dependent setup */
	board_init,			/* basic board dependent setup */
 	serial_init,		/* serial communications setup */
	print_info,
	ici_init,                       /* Init ICI interfaces */
	i2c_init,			/* Init the OpenCores I2C Controller */
	dram_init,			/* DRAM SPD initialization */
	NULL,
};

#ifdef CONFIG_MMC
int mmc_read_bootloader(int dev, int part)
{
	return 0; /* success */
}
#endif /* CONFIG_MMC */

void boot_start(void)
{
	init_fnc_t **init_fnc_ptr;
	for (init_fnc_ptr = init_sequence; *init_fnc_ptr; ++init_fnc_ptr) {
		if ((*init_fnc_ptr)() != 0) {
			hang ();
		}
	}
	
	/* Initialization complete - start the monitor 
	 */
	for (;;) {
		main_loop ();
	}
	
	/* NOTREACHED - no way out of command loop except booting 
	 */
}

void hang (void)
{
	/* call board specific hang function */
	/* board_hang(); */
	
	/* if board_hang() returns, hang here */
	printf("X-Loader hangs\n");
	for (;;);
}
