/*
 * Copyright 2003-2010 Netlogic Microsystems (“Netlogic”). All rights
 * reserved. Redistribution and use in source and binary forms, with 
 * or without modification, are permitted provided that the following 
 * conditions are met:
 *
 *	1. 	Redistributions of source code must retain the above copyright
 *		notice, this list of conditions and the following disclaimer.
 *
 *	2. 	Redistributions in binary form must reproduce the above copyright
 *		notice, this list of conditions and the following disclaimer in
 *		the documentation and/or other materials provided with the
 *		distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY Netlogic Microsystems ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NETLOGIC OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 * --------------------------------#NETL_2#--------------------------------
 */

/* This file has the DRAM programming with
 * added hooks for supporting multiple nodes
 * and channels across dual I2C busses. 
 */

#include <common.h>
#include <asm/arch/cpu.h>

#include "spd.h"
#include "clock.h"
#include "ddr_helper.h"

static const struct ddr_tge_data zero_data =
{
	{ 0, 0,
	  0, 0, 
	  0, 0,
	  0, 0, 
	  0, 0,
	  0, 0,
	  0, 0,
	  0, 0},
	{ 0, 0} 
} ;


void ddr_rmw_reg(volatile u32* mmio, 
		 u32 reg, 
		 u32 val, 
		 u32 mask, 
		 u32 offset) 
{ 
	mmio[reg] = ((mmio[reg] & ~(mask << offset)) | ((val & mask) << offset)); 
}

u32 ddr_rd_reg(volatile u32* mmio, 
	       u32 reg, 
	       u32 mask, 
	       u32 offset) 
{

	return ((mmio[reg] >> offset) & mask); 
}
void ddr_wr_reg(volatile u32* mmio, 
		u32 reg, 
		u32 val) 
{ 
	mmio[reg] = val ; 
}

void ddr_phy_wr_reg(volatile u32* mmio,u32 reg, u32 val) 
{ 
	mmio[DDR_PRG] = (reg     |
			 (2<<12) | 
			 (val << 16) ) ;
	
	while ( (mmio[DDR_PRG] >> 13) & 0x1) {} ;
		     
	return ;
}

u32 ddr_phy_rd_reg(volatile u32* mmio, u32 reg) 
{

	mmio[DDR_PRG] = (reg |
			 (3<<12) ) ;
	
	while ( (mmio[DDR_PRG] >> 13) & 0x1) {} ;
		     
	return (mmio[DDR_PRG] >> 16) ;	
}

void ddr_wr_rx_strben_dly(volatile u32* mmio,int rank,int byte_lane,int coarse_delay, int fine_delay) 
{
	u32 reg_base = DDR_PHY_DQS0_CALDLL_BASE + (byte_lane*64) ;
	ddr_phy_wr_reg(mmio,(reg_base + DDR_PHY_DQS_RK0_RXSEDLL_CTRL + rank),fine_delay);
	ddr_phy_wr_reg(mmio,(reg_base + DDR_PHY_DQS_RK0_RXSEDLY_SEL + rank),(1 << coarse_delay) );
} 
void ddr_wr_rx_cten_dly(volatile u32* mmio,int rank,int byte_lane,int coarse_delay) 
{ 
	u32 reg_base = DDR_PHY_DQS0_CALDLL_BASE + (byte_lane*64) ;
	ddr_phy_wr_reg(mmio,(reg_base + DDR_PHY_DQS_RK0_CTENDLY_SEL + rank), (1<<coarse_delay) );
}
void ddr_wr_rx_strb_dly(volatile u32* mmio,int rank,int byte_lane,int delay) 
{
	u32 reg_base = DDR_PHY_DQS0_CALDLL_BASE + (byte_lane*64) ;
	ddr_phy_wr_reg(mmio,(reg_base + DDR_PHY_DQS_RK0_RXPSDLL_CTRL + rank),delay);
} 
void ddr_wr_rx_datavld_dly(volatile u32* mmio,int rank,int byte_lane,int delay) 
{
	u32 reg_base = DDR_PHY_DQS0_CALDLL_BASE + (byte_lane*64) ;
	ddr_phy_wr_reg(mmio,(reg_base + DDR_PHY_DQS_RK0_RXDVDLY_SEL + rank),delay);
} 
u32 ddr_rd_strb_sample(volatile u32* mmio,int byte_lane) 
{
	u32 reg_base = DDR_PHY_DQS0_CALDLL_BASE + (byte_lane*64) ;
	return ddr_phy_rd_reg(mmio,(reg_base + DDR_PHY_DQS_STRB_EN_SAMPLE) );
} 
u32 ddr_rd_txwl_cdly(volatile u32* mmio,int rank, int byte_lane) 
{
	u32 reg_base = DDR_PHY_DQS0_CALDLL_BASE + (byte_lane*64) ;
	return ddr_phy_rd_reg(mmio,(reg_base + DDR_PHY_DQS_RK0_TXWLDLY_SEL + rank) );	
}
void ddr_wr_txwl_cdly(volatile u32* mmio,int rank, int byte_lane,int delay) 
{
	u32 reg_base = DDR_PHY_DQS0_CALDLL_BASE + (byte_lane*64) ;
	ddr_phy_wr_reg(mmio,(reg_base + DDR_PHY_DQS_RK0_TXWLDLY_SEL + rank),delay);	
}
u32 ddr_rd_txwl_fdly(volatile u32* mmio,int rank, int byte_lane) 
{
	u32 reg_base = DDR_PHY_DQS0_CALDLL_BASE + (byte_lane*64) ;
	return ddr_phy_rd_reg(mmio,(reg_base + DDR_PHY_DQS_RK0_TXWLDLL_CTRL + rank) );	
}
void ddr_wr_txwl_fdly(volatile u32* mmio,int rank, int byte_lane,int delay) 
{
	u32 reg_base = DDR_PHY_DQS0_CALDLL_BASE + (byte_lane*64) ;
	ddr_phy_wr_reg(mmio,(reg_base + DDR_PHY_DQS_RK0_TXWLDLL_CTRL + rank),delay);	
}
void ddr_wr_tx_strb_dly(volatile u32* mmio,int byte_lane,int delay) 
{
	u32 reg_base = DDR_PHY_DQS0_CALDLL_BASE + (byte_lane*64) ;
	ddr_phy_wr_reg(mmio,(reg_base + DDR_PHY_DQS_SHR_TXPSDLL_CTRL),delay);
} 
u32 ddr_rd_pad_drv_ctrl(volatile u32* mmio,int byte_lane) 
{
	u32 reg_base = DDR_PHY_DQS0_CALDLL_BASE + (byte_lane*64) ;
	return ddr_phy_rd_reg(mmio,(reg_base + DDR_PHY_DQS_PAD_DRV_CTRL) );     
}
void ddr_wr_pad_drv_ctrl(volatile u32* mmio,int byte_lane,int val) 
{
	u32 reg_base = DDR_PHY_DQS0_CALDLL_BASE + (byte_lane*64) ;
	ddr_phy_wr_reg(mmio,(reg_base + DDR_PHY_DQS_PAD_DRV_CTRL),val);     
}
void ddr_wr_sraddr_strb_dly(volatile u32* mmio,int delay) 
{
	u32 reg_base = DDR_PHY_SRCMD_CALDLL_BASE ;
	ddr_phy_wr_reg(mmio,(reg_base + DDR_PHY_DQS_SHR_TXPSDLL_CTRL),delay);
} 
void ddr_wr_mraddr_strb_dly(volatile u32* mmio,int delay) 
{
	u32 reg_base = DDR_PHY_MRCMD_CALDLL_BASE  ;
	ddr_phy_wr_reg(mmio,(reg_base + DDR_PHY_DQS_SHR_TXPSDLL_CTRL),delay);
} 
	
void ddr_dll_calibrate(volatile u32* mmio) 
{
	/* There is no way to know when calibration is complete */
	/* As a work around we zero out the scaled DLL elements */
	/* then recalibrate and check for a non-zero result     */
	u32 val = ddr_phy_rd_reg(mmio,DDR_PHY_DQS_SHR_TXPSDLL_CTRL) ;
	
	ddr_phy_wr_reg(mmio,DDR_PHY_DQS_SHR_TXPSDLL_CTRL,(val & 0x7f) | (0x1ff << 7) );

	ddr_rmw_reg(mmio,DDR_CM1,0x1,0x1,21); 

	while ( (ddr_phy_rd_reg(mmio,DDR_PHY_DQS_SHR_TXPSDLL_CTRL) >> 7) == 0x1ff) { }

}
int ddr_rank_vld(struct ddr_params* params,
		 int rank) 
{
	if ( (params->rank_vec & (1 << rank) ) ) {
		return 1 ;
	}
	return 0;
}
	
int ddr_lane_vld(struct ddr_params* params,
		 int rank,
		 int lane)
{
	if (ddr_rank_vld(params,rank) ) {

		if ( (lane < 4) ||
		     ( (lane >= 4) && (lane < 8) && (params->mod_width > 32) ) ||
		     ( (lane == 8) && (params->ecc_dev == 1) ) ) {
			return 1;
		}
	}
	return 0;
}
/* Disable Parallel Issue of various command sequences */
void ddr_disable_pissue(volatile u32* mmio)
{
	ddr_rmw_reg(mmio,DDR_DIS1,0x0,0x1,7) ;
	ddr_rmw_reg(mmio,DDR_DIS2,0x0,0x1,7) ;
	ddr_rmw_reg(mmio,DDR_DIS3,0x0,0x1,7) ;
	ddr_rmw_reg(mmio,DDR_DIS4,0x0,0x1,7) ;
	ddr_rmw_reg(mmio,DDR_DIS5,0x0,0x1,7) ;
	ddr_rmw_reg(mmio,DDR_DIS6,0x0,0x1,7) ;
	ddr_rmw_reg(mmio,DDR_RFS1,0x0,0x1,7) ;
	ddr_rmw_reg(mmio,DDR_RFS2,0x0,0x1,7) ;
	ddr_rmw_reg(mmio,DDR_RFS3,0x0,0x1,7) ;
	ddr_rmw_reg(mmio,DDR_RFS4,0x0,0x1,7) ;
	ddr_rmw_reg(mmio,DDR_SRE1,0x0,0x1,7) ;
	ddr_rmw_reg(mmio,DDR_SRE2,0x0,0x1,7) ;
	ddr_rmw_reg(mmio,DDR_SRE3,0x0,0x1,7) ;
	ddr_rmw_reg(mmio,DDR_SRE4,0x0,0x1,7) ;
	ddr_rmw_reg(mmio,DDR_ZQC1,0x0,0x1,7) ;
	ddr_rmw_reg(mmio,DDR_ZQC2,0x0,0x1,7) ;
	ddr_rmw_reg(mmio,DDR_PDE1,0x0,0x1,7) ;
	ddr_rmw_reg(mmio,DDR_PDE2,0x0,0x1,7) ;
	ddr_rmw_reg(mmio,DDR_PDX1,0x0,0x1,7) ;
	ddr_rmw_reg(mmio,DDR_PDX2,0x0,0x1,7) ;
	ddr_rmw_reg(mmio,DDR_UCS1,0x0,0x1,7) ;
	ddr_rmw_reg(mmio,DDR_UCS2,0x0,0x1,7) ;
	ddr_rmw_reg(mmio,DDR_UCS3,0x0,0x1,7) ;
	ddr_rmw_reg(mmio,DDR_UCS4,0x0,0x1,7) ;
	ddr_rmw_reg(mmio,DDR_UCS5,0x0,0x1,7) ;
	ddr_rmw_reg(mmio,DDR_UCS6,0x0,0x1,7) ;
}

void ddr_init_seq(int node,u8 chan)
{
	volatile u32* mmio = (volatile u32*) dmc_io_mmio(node, chan);

	/* Start Dram Init Sequence */
	mmio[DDR_CM1] = mmio[DDR_CM1] | DDR_START_DRAM_INIT;

#ifndef XLP_SIM
	/* Wait for DMC to complete init */
	while ((mmio[DDR_CM1] & DDR_START_DRAM_INIT) != 0) {};
#endif

	/* Enable Transaction Queue */
	mmio[DDR_CM1] = (mmio[DDR_CM1] |  
			 DDR_ENABLE_TX_QUE) ;
			   
	return;
}
void ddr_ucs_init_seq(volatile u32* mmio)
{
	mmio[DDR_UCS1] = mmio[DDR_DIS1] ;
	mmio[DDR_UCS2] = mmio[DDR_DIS2] ;
	mmio[DDR_UCS3] = mmio[DDR_DIS3] ;
	mmio[DDR_UCS4] = mmio[DDR_DIS4] ;
	mmio[DDR_UCS5] = mmio[DDR_DIS5] ;
	mmio[DDR_UCS6] = mmio[DDR_DIS6] ;

        /* Start user command sequence */
	mmio[DDR_UCSC] = 0x1 ;
	
	/* Check user command sequence accepted */
	while ( (mmio[DDR_UCSC] & 0x1) != 0) {} ;
	/* Check user command sequence completed */
	while ( (mmio[DDR_UCSC] & 0x3) != 0) {} ;	
			   
	return;
}

void ddr_do_wlevel(volatile u32* mmio,u32 rank_vec) 
{
	u32 reg_base ;
	int lane ;
	int rank;

	for (lane=0;lane<8;lane++) {
	  reg_base = DDR_PHY_DQS0_CALDLL_BASE + (lane*64) ;
	  
          /*
	  printf ("WLEDGE: 0x%08x\n",ddr_phy_rd_reg(mmio,reg_base + \
						    DDR_PHY_DQS_SHR_WLED_ALN) ) ;
          */	  
	  ddr_phy_wr_reg(mmio,(reg_base + DDR_PHY_DQS_SHR_WLED_CFG),8) ;
	  /*
	  printf ("WLEDGE_CFG: 0x%08x\n",ddr_phy_rd_reg(mmio,reg_base + \
							DDR_PHY_DQS_SHR_WLED_CFG) ) ;
	  */
	}

	for (rank = 0 ; rank < DDR_NUM_RANKS; rank++) {
		if ( (rank_vec & (1 << rank)) != 0){ /* Rank is populated */

			/* Issue UCS to enter write leveling mode */
			ddr_enter_wrl_mode(mmio, rank);

			/* Trigger leveling on only selected rank. Don't use h/w to issue MRS commands */
			mmio[DDR_WLCTL] = (8 << 8) | ( (1 << rank) << 4) | 0x3;

#ifndef XLP_SIM
			/* Wait for DMC to complete write level */
			while ( (mmio[DDR_WLCTL] & 0x1) != 0) {};
#endif

			/* Issue UCS to exit write leveling mode  */
			ddr_leave_wrl_mode(mmio, rank);	

		}	
	}

	/*
	for (lane=0;lane<9;lane++) {
	  reg_base = DDR_PHY_DQS0_CALDLL_BASE + (lane*64) ;
	  printf ("Lane %d Rank 0 H/W leveling coarse results: 0x%x\n", lane, ddr_phy_rd_reg(mmio,reg_base + DDR_PHY_DQS_RK0_TXWLDLY_SEL) ) ;
	  printf ("Lane %d Rank 1 H/W leveling coarse results: 0x%x\n", lane, ddr_phy_rd_reg(mmio,reg_base + DDR_PHY_DQS_RK1_TXWLDLY_SEL) ) ;
	  printf ("Lane %d Rank 2 H/W leveling coarse results: 0x%x\n", lane, ddr_phy_rd_reg(mmio,reg_base + DDR_PHY_DQS_RK2_TXWLDLY_SEL) ) ;
	  printf ("Lane %d Rank 3 H/W leveling coarse results: 0x%x\n", lane, ddr_phy_rd_reg(mmio,reg_base + DDR_PHY_DQS_RK3_TXWLDLY_SEL) ) ;
	}

	for (lane=0;lane<9;lane++) {
	  reg_base = DDR_PHY_DQS0_CALDLL_BASE + (lane*64) ;
	  printf ("Lane %d Rank 0 H/W leveling fine   results: 0x%x\n", lane, ddr_phy_rd_reg(mmio,reg_base + DDR_PHY_DQS_RK0_TXWLDLL_CTRL) & 0x7f) ;
	  printf ("Lane %d Rank 1 H/W leveling fine   results: 0x%x\n", lane, ddr_phy_rd_reg(mmio,reg_base + DDR_PHY_DQS_RK1_TXWLDLL_CTRL) & 0x7f) ;
	  printf ("Lane %d Rank 2 H/W leveling fine   results: 0x%x\n", lane, ddr_phy_rd_reg(mmio,reg_base + DDR_PHY_DQS_RK2_TXWLDLL_CTRL) & 0x7f) ;
	  printf ("Lane %d Rank 3 H/W leveling fine   results: 0x%x\n", lane, ddr_phy_rd_reg(mmio,reg_base + DDR_PHY_DQS_RK3_TXWLDLL_CTRL) & 0x7f) ;
	}

	for (lane=0;lane<9;lane++) {
	  reg_base = DDR_PHY_DQS0_CALDLL_BASE + (lane*64) ;
	  printf ("Lane %d WLED_FVR: 0x%08x\n", lane, ddr_phy_rd_reg(mmio,reg_base + DDR_PHY_DQS_SHR_WLED_FVR) ) ;
	  printf ("Lane %d WLED_NVR: 0x%08x\n", lane, ddr_phy_rd_reg(mmio,reg_base + DDR_PHY_DQS_SHR_WLED_NVR) ) ;
	  printf ("Lane %d WLED_ALN: 0x%08x\n", lane, ddr_phy_rd_reg(mmio,reg_base + DDR_PHY_DQS_SHR_WLED_ALN) ) ;
	}
	*/
	
	return ;

}
void ddr_disable_dynodt_mode(volatile u32* mmio) 
{
	/* Disable dynamic ODT */
	mmio[DDR_UCS1] = (mmio[DDR_DIS1] & ~(3 << 17) ) ;
	mmio[DDR_UCS2] = (0xf<<28) | (DDR_UCS_NOP<<24) | (0x0<<8) | (0x3<<4) | 1;
	mmio[DDR_UCS3] = (0xf<<28) | (DDR_UCS_NOP<<24) | (0x0<<8) | (0x3<<4) | 1;
	mmio[DDR_UCS4] = (0xf<<28) | (DDR_UCS_NOP<<24) | (0x0<<8) | (0x3<<4) | 1;
	mmio[DDR_UCS5] = (0xf<<28) | (DDR_UCS_NOP<<24) | (0x0<<8) | (0x3<<4) | 1;
	mmio[DDR_UCS6] = (0xf<<28) | (DDR_UCS_NOP<<24) | (0x0<<8) | (0x3<<4) | 1;

	/* Start user command sequence */
	mmio[DDR_UCSC] = 0x1 ;
	
	/* Check user command sequence accepted */
	while ( (mmio[DDR_UCSC] & 0x1) != 0) {} ;
	/* Check user command sequence completed */
	while ( (mmio[DDR_UCSC] & 0x3) != 0) {} ;	
}
void ddr_enable_dynodt_mode(volatile u32* mmio) 
{
	/* Set dynamic ODT to RZQ/4 */
	mmio[DDR_UCS1] = (mmio[DDR_DIS1] & ~(3 << 17) ) | (1 << 17) ;

	mmio[DDR_UCS2] = (0xf<<28) | (DDR_UCS_NOP<<24) | (0x0<<8) | (0x3<<4) | 1;
	mmio[DDR_UCS3] = (0xf<<28) | (DDR_UCS_NOP<<24) | (0x0<<8) | (0x3<<4) | 1;
	mmio[DDR_UCS4] = (0xf<<28) | (DDR_UCS_NOP<<24) | (0x0<<8) | (0x3<<4) | 1;
	mmio[DDR_UCS5] = (0xf<<28) | (DDR_UCS_NOP<<24) | (0x0<<8) | (0x3<<4) | 1;
	mmio[DDR_UCS6] = (0xf<<28) | (DDR_UCS_NOP<<24) | (0x0<<8) | (0x3<<4) | 1;

	/* Start user command sequence */
	mmio[DDR_UCSC] = 0x1 ;
	
	/* Check user command sequence accepted */
	while ( (mmio[DDR_UCSC] & 0x1) != 0) {} ;
	/* Check user command sequence completed */
	while ( (mmio[DDR_UCSC] & 0x3) != 0) {} ;	
}

void ddr_enter_wrl_mode(volatile u32* mmio, int rank) 
{
	u32 value0=0;
	u32 value1=0;
	u32 value2=0;
	u32 value3=0;

	/* Read shadow MRS values for each rank. Set the leveling mode bit 
	   MR1[7] for all ranks. Set the Qoff bit MR1[12] to disable outputs
           for all ranks not being leveled. */
	if (rank == 0x0) {
		value0 = mmio[DDR_RK0SMR1] | (0x1<<7);
		value1 = mmio[DDR_RK1SMR1] | (0x1<<7) | (0x1<<12);
		value2 = mmio[DDR_RK2SMR1] | (0x1<<7) | (0x1<<12);
		value3 = mmio[DDR_RK3SMR1] | (0x1<<7) | (0x1<<12);
	}
	else if (rank == 0x1){
		value0 = mmio[DDR_RK0SMR1] | (0x1<<7) | (0x1<<12);
		value1 = mmio[DDR_RK1SMR1] | (0x1<<7);
		value2 = mmio[DDR_RK2SMR1] | (0x1<<7) | (0x1<<12);
		value3 = mmio[DDR_RK3SMR1] | (0x1<<7) | (0x1<<12);
	}
	else if (rank == 0x2){
		value0 = mmio[DDR_RK0SMR1] | (0x1<<7) | (0x1<<12);
		value1 = mmio[DDR_RK1SMR1] | (0x1<<7) | (0x1<<12);
		value2 = mmio[DDR_RK2SMR1] | (0x1<<7);
		value3 = mmio[DDR_RK3SMR1] | (0x1<<7) | (0x1<<12);
	}
	else if (rank == 0x3){
		value0 = mmio[DDR_RK0SMR1] | (0x1<<7) | (0x1<<12);
		value1 = mmio[DDR_RK1SMR1] | (0x1<<7) | (0x1<<12);
		value2 = mmio[DDR_RK2SMR1] | (0x1<<7) | (0x1<<12);
		value3 = mmio[DDR_RK3SMR1] | (0x1<<7);
	} 
	mmio[DDR_UCS1] = (0x1<<28) | (DDR_UCS_MRS<<24) | (value0<<8) | (0x1<<4) | 1; 
	mmio[DDR_UCS2] = (0x2<<28) | (DDR_UCS_MRS<<24) | (value1<<8) | (0x1<<4) | 1; 
	mmio[DDR_UCS3] = (0x4<<28) | (DDR_UCS_MRS<<24) | (value2<<8) | (0x1<<4) | 1; 
	mmio[DDR_UCS4] = (0x8<<28) | (DDR_UCS_MRS<<24) | (value3<<8) | (0x1<<4) | 1; 
	mmio[DDR_UCS5] = (0xf<<28) | (DDR_UCS_NOP<<24) | (   0x0<<8) | (0x0<<4) | 1;
	mmio[DDR_UCS6] = (0xf<<28) | (DDR_UCS_NOP<<24) | (   0x0<<8) | (0x0<<4) | 1;

	/* Start user command sequence */
	mmio[DDR_UCSC] = 0x1 ;
	
	/* Check user command sequence accepted */
	while ( (mmio[DDR_UCSC] & 0x1) != 0) {} ;
	/* Check user command sequence completed */
	while ( (mmio[DDR_UCSC] & 0x3) != 0) {} ;	
}


void ddr_leave_wrl_mode(volatile u32* mmio, int rank) 
{
	u32 value0=0;
	u32 value1=0;
	u32 value2=0;
	u32 value3=0;

	/* Read shadow MRS values for each rank. Clear the leveling mode bit 
	   MR1[7] for all ranks. Clear the Qoff bit MR1[12] to enable drivers 
           for all ranks. */
	if (rank == 0x0) {
		value0 = (mmio[DDR_RK0SMR1] & 0xef7f) ;
		value1 = (mmio[DDR_RK1SMR1] & 0xef7f) ;
		value2 = (mmio[DDR_RK2SMR1] & 0xef7f) ;
		value3 = (mmio[DDR_RK3SMR1] & 0xef7f) ;
	}
	else if (rank == 0x1){
		value0 = (mmio[DDR_RK0SMR1] & 0xef7f) ;
		value1 = (mmio[DDR_RK1SMR1] & 0xef7f) ;
		value2 = (mmio[DDR_RK2SMR1] & 0xef7f) ;
		value3 = (mmio[DDR_RK3SMR1] & 0xef7f) ;
	}
	else if (rank == 0x2){
		value0 = (mmio[DDR_RK0SMR1] & 0xef7f) ;
		value1 = (mmio[DDR_RK1SMR1] & 0xef7f) ;
		value2 = (mmio[DDR_RK2SMR1] & 0xef7f) ;
		value3 = (mmio[DDR_RK3SMR1] & 0xef7f) ;
	}
	else if (rank == 0x3){
		value0 = (mmio[DDR_RK0SMR1] & 0xef7f) ;
		value1 = (mmio[DDR_RK1SMR1] & 0xef7f) ;
		value2 = (mmio[DDR_RK2SMR1] & 0xef7f) ;
		value3 = (mmio[DDR_RK3SMR1] & 0xef7f) ;
	}
	mmio[DDR_UCS1] = (0x1<<28) | (DDR_UCS_MRS<<24) | (value0<<8) | (0x1<<4) | 1; 
	mmio[DDR_UCS2] = (0x2<<28) | (DDR_UCS_MRS<<24) | (value1<<8) | (0x1<<4) | 1; 
	mmio[DDR_UCS3] = (0x4<<28) | (DDR_UCS_MRS<<24) | (value2<<8) | (0x1<<4) | 1; 
	mmio[DDR_UCS4] = (0x8<<28) | (DDR_UCS_MRS<<24) | (value3<<8) | (0x1<<4) | 1; 
	mmio[DDR_UCS5] = (0xf<<28) | (DDR_UCS_NOP<<24) | (   0x0<<8) | (0x0<<4) | 1;
	mmio[DDR_UCS6] = (0xf<<28) | (DDR_UCS_NOP<<24) | (   0x0<<8) | (0x0<<4) | 1;

	/* Start user command sequence */
	mmio[DDR_UCSC] = 0x1 ;
	
	/* Check user command sequence accepted */
	while ( (mmio[DDR_UCSC] & 0x1) != 0) {} ;
	/* Check user command sequence completed */
	while ( (mmio[DDR_UCSC] & 0x3) != 0) {} ;	
}

void ddr_write_cmr(volatile u32* mmio,u32 reg, u32 val) 
{
	u32 ba ;
	u32 ma ;
	u32 par ;
	u32 cmd ;
	int i ;

	ma = ( (val & 0x3) << 3) | (reg & 0x7) ;
	ba = ( ( (val >> 2) ) | ( (reg >> 3) << 2) ) ;
	par = 0;
	cmd = DDR_UCS_NOP ;

	/* A0 workaround */
	for (i = 0; i < 5; i++) {
		par = par ^ ( (ma>>i) & 0x1) ;
	}
	for (i = 0; i < 3; i++) {
		par = par ^ ( (ba>>i) & 0x1) ;
	}
	
	if (par == 0) {
		cmd = DDR_UCS_MRS ;
	} else {
		cmd = DDR_UCS_NOP ;
	}
	/* End A0 workaround */
		
	mmio[DDR_UCS1] = ( (0xf<<28)   | 
			   (cmd <<24)  | 
			   (ma  <<8)   | 
			   (1   << 7)  | 
			   (ba  <<4)   | 
			   1) ;

	mmio[DDR_UCS2] = (0xf<<28) | (DDR_UCS_NOP<<24) | (0x0<<8) | (0 << 7) | (0x3<<4) | 1;
	mmio[DDR_UCS3] = (0xf<<28) | (DDR_UCS_NOP<<24) | (0x0<<8) | (0 << 7) | (0x3<<4) | 1;
	mmio[DDR_UCS4] = (0xf<<28) | (DDR_UCS_NOP<<24) | (0x0<<8) | (0 << 7) | (0x3<<4) | 1;
	mmio[DDR_UCS5] = (0xf<<28) | (DDR_UCS_NOP<<24) | (0x0<<8) | (0 << 7) | (0x3<<4) | 1;
	mmio[DDR_UCS6] = (0xf<<28) | (DDR_UCS_NOP<<24) | (0x0<<8) | (0 << 7) | (0x3<<4) | 1;

	/* Start user command sequence */
	mmio[DDR_UCSC] = 0x1 ;
	
	/* Check user command sequence accepted */
	while ( (mmio[DDR_UCSC] & 0x1) != 0) {} ;
	/* Check user command sequence completed */
	while ( (mmio[DDR_UCSC] & 0x3) != 0) {} ;

	/* Issue extra NOPs to avoid timing violation */
	mmio[DDR_UCS1] = (0xf<<28) | (DDR_UCS_NOP<<24) | (0x0<<8) | (0 << 7) | (0x3<<4) | 1;
	mmio[DDR_UCS2] = (0xf<<28) | (DDR_UCS_NOP<<24) | (0x0<<8) | (0 << 7) | (0x3<<4) | 1;
	mmio[DDR_UCS3] = (0xf<<28) | (DDR_UCS_NOP<<24) | (0x0<<8) | (0 << 7) | (0x3<<4) | 1;
	mmio[DDR_UCS4] = (0xf<<28) | (DDR_UCS_NOP<<24) | (0x0<<8) | (0 << 7) | (0x3<<4) | 1;
	mmio[DDR_UCS5] = (0xf<<28) | (DDR_UCS_NOP<<24) | (0x0<<8) | (0 << 7) | (0x3<<4) | 1;
	mmio[DDR_UCS6] = (0xf<<28) | (DDR_UCS_NOP<<24) | (0x0<<8) | (0 << 7) | (0x3<<4) | 1;

	/* Start user command sequence */
	mmio[DDR_UCSC] = 0x1 ;	
	
	/* Check user command sequence accepted */
	while ( (mmio[DDR_UCSC] & 0x1) != 0) {} ;
	/* Check user command sequence completed */
	while ( (mmio[DDR_UCSC] & 0x3) != 0) {} ;
}

void ddr_enter_mpr_mode(volatile u32* mmio) 
{
	mmio[DDR_UCS1] = (0xf<<28) | (DDR_UCS_MRS<<24) | (0x4<<8) | (0x3<<4) | 1;
	mmio[DDR_UCS2] = (0xf<<28) | (DDR_UCS_NOP<<24) | (0x0<<8) | (0x3<<4) | 1;
	mmio[DDR_UCS3] = (0xf<<28) | (DDR_UCS_NOP<<24) | (0x0<<8) | (0x3<<4) | 1;
	mmio[DDR_UCS4] = (0xf<<28) | (DDR_UCS_NOP<<24) | (0x0<<8) | (0x3<<4) | 1;
	mmio[DDR_UCS5] = (0xf<<28) | (DDR_UCS_NOP<<24) | (0x0<<8) | (0x3<<4) | 1;
	mmio[DDR_UCS6] = (0xf<<28) | (DDR_UCS_NOP<<24) | (0x0<<8) | (0x3<<4) | 1;

	/* Start user command sequence */
	mmio[DDR_UCSC] = 0x1 ;
	
	/* Check user command sequence accepted */
	while ( (mmio[DDR_UCSC] & 0x1) != 0) {} ;
	/* Check user command sequence completed */
	while ( (mmio[DDR_UCSC] & 0x3) != 0) {} ;	
}

void ddr_leave_mpr_mode(volatile u32* mmio) 
{

	mmio[DDR_UCS1] = (0xf<<28) | (DDR_UCS_MRS<<24) | (0x0<<8) | (0x3<<4) | 1;
	mmio[DDR_UCS2] = (0xf<<28) | (DDR_UCS_NOP<<24) | (0x0<<8) | (0x3<<4) | 1;
	mmio[DDR_UCS3] = (0xf<<28) | (DDR_UCS_NOP<<24) | (0x0<<8) | (0x3<<4) | 1;
	mmio[DDR_UCS4] = (0xf<<28) | (DDR_UCS_NOP<<24) | (0x0<<8) | (0x3<<4) | 1;
	mmio[DDR_UCS5] = (0xf<<28) | (DDR_UCS_NOP<<24) | (0x0<<8) | (0x3<<4) | 1;
	mmio[DDR_UCS6] = (0xf<<28) | (DDR_UCS_NOP<<24) | (0x0<<8) | (0x3<<4) | 1;
	
	/* Start user command sequence */
	mmio[DDR_UCSC] = 0x1 ;
	
	/* Check user command sequence accepted */
	while ( (mmio[DDR_UCSC] & 0x1) != 0) {} ;
	/* Check user command sequence completed */
	while ( (mmio[DDR_UCSC] & 0x3) != 0) {} ;
}

void ddr_tge_setup_data(volatile u32* mmio,
			const struct  ddr_tge_data* data) 
{	
	int i;
	for (i = 0; i < 16; i++) {
		mmio[DDR_TGED1 + i] = data->data[i];
	}
	mmio[DDR_TGECB1] = data->cb[0];
	mmio[DDR_TGECB2] = data->cb[1];		
}


void ddr_tge_txn(volatile u32* mmio,
		 __u64 address, 
		 u32 stride,
		 u32 cnt,
		 u8 read) 
{		
	int timeout = 0xffffff ;

	/* Number of transactions */
	mmio[DDR_TGECNT] = cnt ;	
	
        /* Setup Address + stride(64 byte) Registers*/
	mmio[DDR_TGEA1]  = (address & 0xffffffffULL) ;
	mmio[DDR_TGEA2]  = ( (stride << 8) |
			     ( (address >> 32) & 0xff) ) ; 
	
	/* Clear mismatch vector */
	if (read) {
		mmio[DDR_TGE_DMVECTLO] = 0;
		mmio[DDR_TGE_DMVECTHI] = 0;
		mmio[DDR_TGE_CBMVECT]  = 0;
	}

	/* Start */
	mmio[DDR_TGECON] = ( (read << 0) |
			     (1    << 1) ) ;

	while ((mmio[DDR_TGECON] & 0x2) != 0) {}
	
	/* 
	 * Work around for incorrect completion indication
	 */
	if (!read) {		
		/* Poll until transaction queue is empty */
		while ( ( (mmio[DDR_CST] & (1<<7) ) == 0) && (timeout != 0) ) {
			timeout-- ;
			if (timeout == 0) {
				printf ("TGE write transaction timeout!\n") ;
			}
		}
	} else {
		/* Poll until all reads have returned */
		while ( (cnt != mmio[DDR_TGE_RDRETCNT]) && (timeout != 0) ) {
			timeout-- ;
			if (timeout == 0) {
				printf ("TGE read transaction timeout!\n") ;
			}
		}
	}	
}

void ddr_tge_data_check(volatile u32* mmio,
			int rank,
			struct ddr_params* params,
			u8 *byte_pass,
			u8 *any_pass,
			u8 *all_pass) 
{
	int lane ;
	int byte ;
	u32 err_vec[3] ;
	u8  lane_fail ;
	int lane_cnt ;

	err_vec[0]  =  mmio[DDR_TGE_DMVECTLO] ;
	err_vec[1]  =  mmio[DDR_TGE_DMVECTHI] ;
	err_vec[2]  =  mmio[DDR_TGE_CBMVECT] ;
	
	*any_pass = 0;
	*all_pass = 0;
	lane_fail = 0;
	
	for (lane = 0; lane < DDR_NUM_BYTE_LANES; lane++) {
		byte_pass[lane] = 1;
	}

	lane_cnt = params->mod_width/8 ;

	for (byte = 0; byte < 64; byte++) {
		/* Adjust for big endian format of lanes
		 */
		byte_pass[lane_cnt - (byte%lane_cnt) - 1] &= ( ( (err_vec[byte/32] >> (byte%32) ) & 0x1) == 0) ;	
	}

	if (params->ecc_dev) {
		byte_pass[8] = ( (err_vec[2] & 0xff) == 0) ;
	} else {
		byte_pass[8] = 1;
	}

	for (lane = 0; lane < DDR_NUM_BYTE_LANES; lane++) {
		if (ddr_lane_vld(params,rank,lane) ) {
			if (byte_pass[lane] == 1) {
				*any_pass = 1;
			} else {
				lane_fail = 1;
			}
		}
	}
	if (lane_fail == 0)
		*all_pass = 1 ;
}

void ddr_tge_memclear(volatile u32* mmio,__u64 address, __u64 size)
{
	u32 cnt ;

	ddr_tge_setup_data(mmio,&zero_data) ;
	
	cnt = (u32) (size >> 5) ;
	ddr_tge_txn(mmio,address,64,cnt,0) ;
}
