/*
 * Copyright 2003-2010 Netlogic Microsystems (“Netlogic”). All rights
 * reserved. Redistribution and use in source and binary forms, with 
 * or without modification, are permitted provided that the following 
 * conditions are met:
 *
 *	1. 	Redistributions of source code must retain the above copyright
 *		notice, this list of conditions and the following disclaimer.
 *
 *	2. 	Redistributions in binary form must reproduce the above copyright
 *		notice, this list of conditions and the following disclaimer in
 *		the documentation and/or other materials provided with the
 *		distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY Netlogic Microsystems ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NETLOGIC OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 * --------------------------------#NETL_2#--------------------------------
 */
#ifndef __DDR_HELPER_H__
#define __DDR_HELPER_H__

#include <common.h>
/* DDR3 controller paramters */
struct ddr_params
{
        /* Module/DIMM parameters */
        uint32_t num_ranks ;
        uint32_t rank_vec ;
        uint32_t quad_rank ;
        uint32_t registered;
        uint32_t mirrored;
        uint32_t mod_width ;

        /* Device parameters */
        uint32_t row_bits ;
        uint32_t col_bits ;
        uint32_t dev_width ;
        uint32_t ecc_dev ;
        uint32_t vdd135_cap ;
        uint32_t rzq_div6;
        uint32_t rzq_div7;
        uint32_t node_density_mb ; /* In Megabits */

        /* Device timing parameters in MTB units */
        uint32_t tck_min ;
        uint32_t taa_min ;
        uint32_t twr_min ;
        uint32_t trcd_min ;
        uint32_t trrd_min ;
        uint32_t trp_min ;
        uint32_t twtr_min ;
        uint32_t trtp_min ;

        uint32_t tfaw_min ;
        uint32_t tras_min ;
        uint32_t trc_min ;
        uint32_t trfc_min ;
        uint32_t cas_vec ;
        uint32_t tref_min ; /* Always 7.8us */

        /* Time units */
        uint32_t ftb_fs ;
        uint32_t mtb_ps ;
}  ;

struct ddr_tge_data {
        u32 data[16];
        u32 cb[2] ;
} ;

struct ddr_params ddr_cntrl[DDR_NUM_CHANNELS];
u32 chan_mem_size[MAX_NODES][DDR_NUM_CHANNELS];

/* DDR PHY register access */
void ddr_phy_wr_reg(volatile u32* mmio,u32 reg, u32 val);
u32 ddr_phy_rd_reg(volatile u32* mmio, u32 reg);


/* DDR Controller register access */
void ddr_rmw_reg(volatile u32* mmio, 
			u32 reg, u32 val, u32 mask, u32 offset);
void ddr_wr_reg(volatile u32* mmio, 
		       u32 reg, u32 val);
u32 ddr_rd_reg(volatile u32* mmio, 
		      u32 reg, u32 mask, u32 offset) ;

/* Helper DDR register access functions */
#define ddr_wr_tccd(mmio,val)      ddr_rmw_reg(mmio,DDR_BP1,val,0x1f,0)
#define ddr_wr_treg(mmio,val)      ddr_rmw_reg(mmio,DDR_BP1,val,0x1f,16)   
#define ddr_wr_adddly_rl(mmio,val) ddr_rmw_reg(mmio,DDR_BP1,val,0x1f,24)  
#define ddr_wr_trc(mmio,val)       ddr_rmw_reg(mmio,DDR_BP2,val,0xff,0) 
#define ddr_wr_trcd(mmio,val)      ddr_rmw_reg(mmio,DDR_BP2,val,0x1f,8)
#define ddr_wr_trrd(mmio,val)      ddr_rmw_reg(mmio,DDR_BP2,val,0x1f,16)
#define ddr_wr_tfaw(mmio,val)      ddr_rmw_reg(mmio,DDR_BP2,val,0x3f,24)
#define ddr_wr_tras(mmio,val)      ddr_rmw_reg(mmio,DDR_BP3,val,0xff,0)     
#define ddr_wr_trfc(mmio,val)      ddr_rmw_reg(mmio,DDR_BP4,val,0xffff,0)   
#define ddr_wr_tref(mmio,val)      ddr_rmw_reg(mmio,DDR_BP4,val,0xffff,16)      
#define ddr_wr_trtp(mmio,val)      ddr_rmw_reg(mmio,DDR_BP5,val,0x1f,0)    
#define ddr_wr_trp(mmio,val)       ddr_rmw_reg(mmio,DDR_BP5,val,0x1f,8)   
#define ddr_wr_twtr(mmio,val)      ddr_rmw_reg(mmio,DDR_BP5,val,0x1f,16)
#define ddr_wr_txpr(mmio,val)      ddr_rmw_reg(mmio,DDR_BP8,val,0xfff,16) 
#define ddr_wr_txs(mmio,val)       ddr_rmw_reg(mmio,DDR_BP13,val,0xfff,4) 
#define ddr_wr_twlo(mmio,val)      ddr_rmw_reg(mmio,DDR_BP15,val,0x3f,16) 
#define ddr_wr_twldqsen(mmio,val)  ddr_rmw_reg(mmio,DDR_BP15,val,0x3f,0) 
#define ddr_wr_tfawlim(mmio,val)   ddr_rmw_reg(mmio,DDR_BP5,val,0x1f,24) 
#define ddr_wr_banklsb(mmio,val)   ddr_rmw_reg(mmio,DDR_AP1,val,0x3f,0) 
#define ddr_wr_ranklsb(mmio,val)   ddr_rmw_reg(mmio,DDR_AP1,val,0x3f,16)  
#define ddr_wr_rankbits(mmio,val)  ddr_rmw_reg(mmio,DDR_AP1,val,0x3,24) 
#define ddr_wr_collsb(mmio,val)    ddr_rmw_reg(mmio,DDR_AP2,val,0x3f,0)  
#define ddr_wr_colbits(mmio,val)   ddr_rmw_reg(mmio,DDR_AP2,val,0x1f,8) 
#define ddr_wr_rowlsb(mmio,val)    ddr_rmw_reg(mmio,DDR_AP2,val,0x3f,16) 
#define ddr_wr_rowbits(mmio,val)   ddr_rmw_reg(mmio,DDR_AP2,val,0x1f,24) 
#define ddr_wr_cs0rnkid(mmio,val)  ddr_rmw_reg(mmio,DDR_AP3,val,0x3,0)  
#define ddr_wr_cs1rnkid(mmio,val)  ddr_rmw_reg(mmio,DDR_AP3,val,0x3,4)  
#define ddr_wr_cs2rnkid(mmio,val)  ddr_rmw_reg(mmio,DDR_AP3,val,0x3,8)  
#define ddr_wr_cs3rnkid(mmio,val)  ddr_rmw_reg(mmio,DDR_AP3,val,0x3,12)  
#define ddr_wr_mirror(mmio,val)    ddr_rmw_reg(mmio,DDR_AP3,val,0xf,28) 
#define ddr_wr_ntaddr(mmio,val)    ddr_rmw_reg(mmio,DDR_CM0,val,0x3,2) 
#define ddr_wr_ntcsdly(mmio,val)   ddr_rmw_reg(mmio,DDR_CM0,val,0x3,4)   
#define ddr_wr_bl16(mmio,val)      ddr_rmw_reg(mmio,DDR_CM0,val,0x1,6) 
#define ddr_enable_ecc(mmio)       ddr_rmw_reg(mmio,DDR_CM1,0xf,0xf,16) 
#define ddr_wr_mr0cas(mmio,val)    ddr_rmw_reg(mmio,DDR_DIS4,\
                                               ( ( (val&0xe) << 1) | (val&0x1) ),\
                                               0x1d,10)  
#define ddr_wr_mr0twr(mmio,val)         ddr_rmw_reg(mmio,DDR_DIS4,val,0x7,17) 
#define ddr_wr_mr2cwl(mmio,val)         ddr_rmw_reg(mmio,DDR_DIS1,val,0x7,11)  
#define ddr_wr_mr3mpr(mmio,val)         ddr_rmw_reg(mmio,DDR_DIS2,val,0x7,8)  
#define ddr_wr_mr1rzqd7(mmio,val)       ddr_rmw_reg(mmio,DDR_DIS3,val,0x1,(8 + 1) )
#define ddr_wr_mr1al(mmio,val)          ddr_rmw_reg(mmio,DDR_DIS3,val,0x3,(8 + 3) )
#define ddr_wr_dqs_prelaunch(mmio,val) 	ddr_rmw_reg(mmio,DDR_BP3,val,0x7,16)
#define ddr_wr_wrwr_dfrk_ta(mmio,val) 	ddr_rmw_reg(mmio,DDR_BP3,val,0xf,8) 
#define ddr_wr_rdrd_dfrk_ta(mmio,val) 	ddr_rmw_reg(mmio,DDR_BP6,val,0xf,0) 
#define ddr_wr_rdwr_smrk_ta(mmio,val) 	ddr_rmw_reg(mmio,DDR_BP6,val,0xf,8) 
#define ddr_wr_rdwr_dfrk_ta(mmio,val) 	ddr_rmw_reg(mmio,DDR_BP6,val,0xf,16) 
#define ddr_wr_wrrd_smrk_ta(mmio,val) 	ddr_rmw_reg(mmio,DDR_BP6,val,0xf,24) 
#define ddr_wr_wrrd_dfrk_ta(mmio,val) 	ddr_rmw_reg(mmio,DDR_BP1,val,0xf,8) 
#define ddr_rd_wrwr_dfrk_ta(mmio) 	ddr_rd_reg(mmio,DDR_BP3,0xf,8) 
#define ddr_rd_rdrd_dfrk_ta(mmio) 	ddr_rd_reg(mmio,DDR_BP6,0xf,0) 
#define ddr_rd_rdwr_smrk_ta(mmio) 	ddr_rd_reg(mmio,DDR_BP6,0xf,8) 
#define ddr_rd_rdwr_dfrk_ta(mmio) 	ddr_rd_reg(mmio,DDR_BP6,0xf,16)
#define ddr_rd_wrrd_smrk_ta(mmio) 	ddr_rd_reg(mmio,DDR_BP6,0xf,24) 
#define ddr_rd_wrrd_dfrk_ta(mmio) 	ddr_rd_reg(mmio,DDR_BP1,0xf,8) 
#define ddr_force_term(mmio)            ddr_rmw_reg(mmio,DDR_CM1,0x3,0x3,24)
#define ddr_unforce_term(mmio)          ddr_rmw_reg(mmio,DDR_CM1,0x0,0x3,24)
#define ddr_disable_autocal(mmio)       ddr_rmw_reg(mmio,DDR_CM1,0x0,0x1,20)
#define ddr_enable_autocal(mmio)        ddr_rmw_reg(mmio,DDR_CM1,0x1,0x1,20)
#define ddr_disable_autoref(mmio)       ddr_rmw_reg(mmio,DDR_CM1,0x0,0x1,4)
#define ddr_enable_autoref(mmio)        ddr_rmw_reg(mmio,DDR_CM1,0x1,0x1,4)
#define ddr_enable_pwrdown(mmio)        ddr_rmw_reg(mmio,DDR_CM1,0x1,0x1,5)
#define ddr_enable_selfref(mmio)        ddr_rmw_reg(mmio,DDR_CM1,0x1,0x1,6)

#define ddr_phy_wr_pad_drvctl(mmio,lane,val) \
	ddr_phy_wr_reg(mmio,(DDR_PHY_DQS0_CALDLL_BASE + (lane*64) + DDR_PHY_DQS_PAD_DRV_CTRL),val)
#define ddr_phy_rd_pad_drvctl(mmio,lane) \
	ddr_phy_rd_reg(mmio,(DDR_PHY_DQS0_CALDLL_BASE + (lane*64) + DDR_PHY_DQS_PAD_DRV_CTRL) )
#define ddr_phy_wr_pad_termctl(mmio,lane,val) \
	ddr_phy_wr_reg(mmio,(DDR_PHY_DQS0_CALDLL_BASE + (lane*64) + DDR_PHY_DQS_PAD_TERM_CTRL),val)
#define ddr_phy_rd_pad_termctl(mmio,lane) \
	ddr_phy_rd_reg(mmio,(DDR_PHY_DQS0_CALDLL_BASE + (lane*64) + DDR_PHY_DQS_PAD_TERM_CTRL) )
#define ddr_phy_wr_pad_drvscale(mmio,lane,val) \
	ddr_phy_wr_reg(mmio,(DDR_PHY_DQS0_CALDLL_BASE + (lane*64) + DDR_PHY_DQS_PAD_DRV_SCALE),val)
#define ddr_phy_rd_pad_drvscale(mmio,lane) \
	ddr_phy_rd_reg(mmio,(DDR_PHY_DQS0_CALDLL_BASE + (lane*64) + DDR_PHY_DQS_PAD_DRV_SCALE) )
#define ddr_phy_wr_pad_termscale(mmio,lane,val) \
	ddr_phy_wr_reg(mmio,(DDR_PHY_DQS0_CALDLL_BASE + (lane*64) + DDR_PHY_DQS_PAD_TERM_SCALE),val)
#define ddr_phy_rd_pad_termscale(mmio,lane) \
	ddr_phy_rd_reg(mmio,(DDR_PHY_DQS0_CALDLL_BASE + (lane*64) + DDR_PHY_DQS_PAD_TERM_SCALE) )
#define ddr_phy_rd_pad0_ctl(mmio,reg) \
	ddr_phy_rd_reg(mmio,(DDR_PHY_PAD0_CAL_BASE + reg) )
#define ddr_phy_rd_pad1_ctl(mmio,reg) \
	ddr_phy_rd_reg(mmio,(DDR_PHY_PAD1_CAL_BASE + reg) )
#define ddr_phy_rd_pad2_ctl(mmio,reg) \
	ddr_phy_rd_reg(mmio,(DDR_PHY_PAD2_CAL_BASE + reg) )
#define ddr_phy_wr_pad0_ctl(mmio,reg,val) \
	ddr_phy_rd_reg(mmio,(DDR_PHY_PAD0_CAL_BASE + reg),val )
#define ddr_phy_wr_pad1_ctl(mmio,reg,val) \
	ddr_phy_rd_reg(mmio,(DDR_PHY_PAD1_CAL_BASE + reg),val )
#define ddr_phy_wr_pad2_ctl(mmio,reg,val) \
	ddr_phy_rd_reg(mmio,(DDR_PHY_PAD2_CAL_BASE + reg),val )


u32 ddr_rd_pad_drv_ctrl(volatile u32* mmio,int byte_lane) ;
void ddr_wr_pad_drv_ctrl(volatile u32* mmio,int byte_lane,int val) ;	
void ddr_wr_rx_strben_dly(volatile u32* mmio,int rank,int byte_lane,int coarse_delay, int fine_delay) ;
void ddr_wr_rx_cten_dly(volatile u32* mmio,int rank,int byte_lane,int coarse_delay) ;
void ddr_wr_rx_strb_dly(volatile u32* mmio,int rank,int byte_lane,int delay) ;
void ddr_wr_rx_datavld_dly(volatile u32* mmio,int rank,int byte_lane,int delay) ;
u32 ddr_rd_strb_sample(volatile u32* mmio,int byte_lane) ;
void ddr_dll_calibrate(volatile u32* mmio) ;
u32 ddr_rd_txwl_cdly(volatile u32* mmio,int rank, int byte_lane) ;
void ddr_wr_txwl_cdly(volatile u32* mmio,int rank, int byte_lane,int delay) ;
u32 ddr_rd_txwl_fdly(volatile u32* mmio,int rank, int byte_lane) ;
void ddr_wr_txwl_fdly(volatile u32* mmio,int rank, int byte_lane,int delay) ;
void ddr_wr_tx_strb_dly(volatile u32* mmio,int byte_lane,int delay) ;
void ddr_disable_dynodt_mode(volatile u32* mmio) ;
void ddr_enable_dynodt_mode(volatile u32* mmio) ;
void ddr_do_wlevel(volatile u32* mmio,u32 rank_vec) ; 
void ddr_wr_sraddr_strb_dly(volatile u32* mmio,int delay) ;
void ddr_wr_mraddr_strb_dly(volatile u32* mmio,int delay) ;
int  ddr_lane_vld(struct ddr_params* params,int rank, int lane) ;
int  ddr_rank_vld(struct ddr_params* params,int rank) ;
void ddr_disable_pissue(volatile u32* mmio);
void ddr_write_cmr(volatile u32* mmio,u32 reg, u32 val)  ;

void ddr_init_seq(int node,u8 chan) ;
void ddr_enter_mpr_mode(volatile u32* mmio) ;
void ddr_leave_mpr_mode(volatile u32* mmio) ;
void ddr_enter_wrl_mode(volatile u32* mmio, int rank) ; 
void ddr_leave_wrl_mode(volatile u32* mmio, int rank) ;
void ddr_ucs_init_seq(volatile u32* mmio) ;

/* TGE (Transaction Generator Functions) */
void ddr_tge_setup_data(volatile u32* mmio,const struct ddr_tge_data* data) ;
void ddr_tge_txn(volatile u32* mmio,u64 address, u32 cnt, u32 stride,u8 read);
void ddr_tge_memclear(volatile u32* mmio,u64 address, u64 size) ;
void ddr_tge_memtest(volatile u32* mmio,u32* err_vec_hi,u32* err_vec_lo,u32* err_vec_ecc);
void ddr_tge_data_check(volatile u32* mmio,
			int rank,
			struct ddr_params* params,
			u8 *byte_pass,
			u8 *any_pass,
			u8 *all_pass) ;

#endif
