/*
 * Copyright 2003-2010 Netlogic Microsystems (“Netlogic”). All rights
 * reserved. Redistribution and use in source and binary forms, with 
 * or without modification, are permitted provided that the following 
 * conditions are met:
 *
 *	1. 	Redistributions of source code must retain the above copyright
 *		notice, this list of conditions and the following disclaimer.
 *
 *	2. 	Redistributions in binary form must reproduce the above copyright
 *		notice, this list of conditions and the following disclaimer in
 *		the documentation and/or other materials provided with the
 *		distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY Netlogic Microsystems ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NETLOGIC OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 * --------------------------------#NETL_2#--------------------------------
 */
#ifndef __DDR_LEVELING_H__
#define __DDR_LEVELING_H__

#include <common.h>

#define RDLVL_FSM_PREAMBLE 0
#define RDLVL_FSM_EDGE1    1
#define RDLVL_FSM_EDGE2    2
#define RDLVL_FSM_DONE     3
#define RDLVL_MIN_EDGE_CNT 32
#define RDLVL_MAX_EDGE_CNT 96 
#define RDLVL_UNDEF_REGION_CNT 8
#define RDLVL_FDLY_STRIDE  1

 
/* DDR Read Level State */
struct rdlvl_state {
	u8 state ;
	int smpl_cnt ;
	u8 addr_rdly ;
	u8 cdly ;
	u8 fdly ;
	u8 lane ;
} ;
#define EDGE_FSM_FAIL 0
#define EDGE_FSM_PASS 1
#define EDGE_FSM_DONE 2
#define EDGE_MIN_CNT  8
/* Edge State */
struct edge_state {
	u8 state ;
	u8 first ;
	u8 last ;
	int cnt;
} ;
#define MIN_SRADDR_WIN 32

/* DDR Phy byte controls */
struct byte_phy {	
	/* Coarse write leveling delay (1/2 cycle incrmements) */
	u8 tx_wl_cdly ;
	/* Fine write leveling delay (1/128 cycle increments, max 64) */
	u8 tx_wl_fdll ;

	/* Write phase shift DLL (1/128 cycle increments)*/
	u8 tx_phase_dll ;
	
	/* Coarse read strobe enable delay (1/2 cycle incrmements) */
	u8 rx_strb_cdly ;
	/* Fine read leveling delay (1/128 cycle increments, max 64) */
	u8 rx_strb_fdll ;
	/* Read byte phase shift DLL (1/128 cycle increments)*/
	u8 rx_byte_phase_dll ;
	/* Read bit shift DLL (1/128 cycle increments, max 32)*/
	u8 rx_bit_phase_dll[8];
      
} ;

/* DDR Phy rank controls */
struct rank_phy {
	struct byte_phy bphy[DDR_NUM_BYTE_LANES] ;	
};

/* DDR Phy channel controls */
struct chan_phy {

	/* Additional address read delay due to board, pad, etc...*/
	u8 addr_rdly ;
	/* Address command DLL settings for single rank load(1/128 cycle increments) */
	u8 addr_srcmd_dll ;
	/* Address command DLL settings for multiple rank load(1/128 cycle increments) */
	u8 addr_mrcmd_dll ;
	/* Address mode (4T,3T,2T,1T) */
	u8 addr_mode ;
	/* Dqs prelaunch value */
	u8 tx_dqs_prelaunch ;

	struct rank_phy rphy[DDR_NUM_RANKS] ;
};

/* Read leveling */
extern int ddr_read_level(int node,
			   int chan,
			   struct ddr_params* params) ;
/* Write leveling */
extern void ddr_write_level(int node,
			     int chan,
			     struct ddr_params* params) ;
/* Address/Cmd Mode adjust*/ 
extern void ddr_addrcmd_align(int node,
			       int chan,
			       struct ddr_params* params) ;

/* Adjust read-write turnaround */ 
extern void ddr_adj_turnaround(int node,
			       int chan,
			       struct ddr_params* params) ;

/* Print leveling info*/ 
extern void ddr_print_linfo(int node,
			    int chan,
			    struct ddr_params* params) ;

#endif
