/*
 * Copyright 2003-2010 Netlogic Microsystems (Netlogic). All rights
 * reserved. Redistribution and use in source and binary forms, with 
 * or without modification, are permitted provided that the following 
 * conditions are met:
 *
 *	1. 	Redistributions of source code must retain the above copyright
 *		notice, this list of conditions and the following disclaimer.
 *
 *	2. 	Redistributions in binary form must reproduce the above copyright
 *		notice, this list of conditions and the following disclaimer in
 *		the documentation and/or other materials provided with the
 *		distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY Netlogic Microsystems ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NETLOGIC OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 * --------------------------------#NETL_2#--------------------------------
 */
#include <asm/asm.h>
#include <asm/arch/cpu.h>
#include <ns16550.h>

LEAF(uart_init)
	
	li		t0, CPU_MMIO_OFFSET(0,UART0)
	
	li		t2, (UART_LCR_DLAB | UART_LCR_8N1)
	sw		t2, (UART_LCR << 2)(t0)
	
	li		t1, UART_DLL_BR
	sw		t1, (UART_DLL << 2)(t0)
	li		t1, UART_DLM_BR
	sw		t1, (UART_DLM << 2)(t0)
	
	li		t2, UART_LCR_8N1
	sw		t2, (UART_LCR << 2)(t0)	
	
	jr	ra
	nop

END(uart_init)

.macro	outbyte	c, reg0, reg1
	li		\reg0, CPU_MMIO_OFFSET(0,UART0)
	/* check for Tx FIFO empty */
11:	lw		\reg1, (UART_LSR << 2)(\reg0)
	andi	\reg1, 0x20
	beqz	\reg1, 11b
	nop

	li		\reg1, 10
	bne		\c, \reg1, 12f
	nop
	/* output linefeed '\r' */
	li		\reg1, 13
	sw		\reg1, (UART_THR << 2)(\reg0)	
12:			
	sw		\c, (UART_THR << 2)(\reg0)
.endm

	/* nibble_counter, loop_counter, shift */
.macro print_hex64 val, shift, i, temp0, temp1
	move	\i, zero
23:	li		\temp0, 60
	sll		\shift, \i, 2
	subu	\shift, \temp0, \shift
	dsra	\shift, \val, \shift
	andi	\shift, 0x0f
	slti	\temp0, \shift, 10
	beqz	\temp0, 21f
	nop
	/* nibble < 10 */
	addiu	\shift, 48
	b		22f
	nop
	/* nibble >= 10 */
21:	addiu	\shift, 55
	
22:	outbyte	\shift, \temp0, \temp1

	addiu	\i, 1
	slti	\temp0, \i, 16
	bnez	\temp0, 23b
	nop
.endm
	
	/* This is really a puts */
LEAF(printk)
	move	t4, a1

1:	lb		t0, 0(a0)
	beqz	t0, 9f
	nop

	/* format marker */
	li		t1, '%'
	bne		t0, t1, 2f
	nop

	/* skip the format char */
	addiu	a0, a0, 1
	lb		t0, 0(a0)
	beqz	t0, 9f
	nop
	/* for now, always print the a1 arg */
	print_hex64 a1, t0, t1, t2, t3
	b		3f
	nop	
2:			
	outbyte	t0, t1, t2
3:		
	addiu	a0, a0, 1
	b		1b
	nop
9:	jr		ra	
	nop
END(printk)
